package cn.com.ctop.job.kuaishou.controller;

import cn.com.ctop.job.kuaishou.data.entity.CompassBindAdvertisers;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouAccountReportHourlyMapper;
import cn.com.ctop.job.kuaishou.data.service.CompassBindAdvertisersService;
import cn.com.ctop.job.kuaishou.data.service.IAccountWarningService;
import cn.com.ctop.job.kuaishou.data.service.IAdUnitReportService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportHourlyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouCampaignReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouMaterialVideoReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouUnitAudienceAgeReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouUnitReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouUnitReportHourlyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouVideoListService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import cn.com.ctop.job.kuaishou.data.service.impl.AdUnitReportServiceImpl;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.util.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * zian Y
 * 2022/2/9
 **/

@RestController
@RequestMapping("/kuaishou/kuaishouController")
@Api(tags = "快手")
@Slf4j
public class KuaishouController {
    @Autowired
    private IOauthTokenService tokenService;

    @Autowired
    private IKuaishouVideoListService videoListService;

    @Autowired
    private IAdUnitReportService adUnitReportService;

    @Autowired
    private IKuaishouUnitReportHourlyService unitReportHourlyService;

    @Autowired
    private IKuaishouAccountReportDailyService kuaishouAccountReportDailyService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(5);
    private static ExecutorService hourlyExecutorService = Executors.newFixedThreadPool(10);


    @ApiOperation(value = "根据账户获取视频素材-账户下全部视频素材", notes = "根据账户获取视频素材-账户下全部视频素材")
    @GetMapping(value = "/getKuaishouVideoByAccountId")
    public void getKuaishouVideoByAccountId() {
        adUnitReportService.getAgentReport();
    }


    @Autowired
    private IKuaishouUnitAudienceAgeReportDailyService unitReportDailyService;


    @ApiOperation(value = "根据账户获取视频素材-账户下全部视频素材", notes = "根据账户获取视频素材-账户下全部视频素材")
    @GetMapping(value = "/KuaishouUnitAudienceReport")
    public void KuaishouUnitAudienceReportJob(String date) throws Exception {

        List<JSONObject> tokens = tokenService.getByProjectId(458L);    //淘特项目
//        String date = DateUtils.getAnotherDay("yyyy-MM-dd", DateUtils.getNowDate("yyyy-MM-dd"), -1);
        Date enDate = DateUtils.addDay(DateUtils.getNowDate(), -1);
        for (JSONObject t : tokens) {
            Long accountId = t.getLong("accountId");
            String token = t.getString("accessToken");
            if (Check.isNull(token)) {
                log.error("此账户未获取到相关token，accountId：{}", accountId);
                continue;
            }
            //过期跳过
            if (enDate.after(t.getDate("accessTokenExpiresIn"))) {
                continue;
            }
            //广告计划 ID 集
            List<JSONObject> campaignArray = unitReportDailyService.selectPlanIdsByAccountIdAndTime(accountId, date);
            if (Check.isNull(campaignArray)) {
                log.error("未查询到计划ID");
                continue;
            }
            //广告组 ID 集
            for (JSONObject obj : campaignArray) {
                Long campaignId = obj.getLong("campaignId");
                List<Long> unitIds = unitReportDailyService.selectUnitIdsByAccountIdAndTime(accountId, campaignId, date);
                if (Check.isNull(unitIds)) {
                    continue;
                }
                if (unitIds.size() > 59) {
                    List<List<Long>> unitIdArray = Lists.partition(unitIds, 59);
                    for (List<Long> list : unitIdArray) {
                        unitReportDailyService.standardCrowdAnalysisReport(accountId, token, date, campaignId, list);
                    }
                } else {
                    unitReportDailyService.standardCrowdAnalysisReport(accountId, token, date, campaignId, unitIds);
                }
            }
        }
    }


    private static ExecutorService dailyFundExecutorService = Executors.newFixedThreadPool(3);

    @GetMapping(value = "/kuaishouFundDailyFlowsAllAccount")
    public void kuaishouFundDailyFlowsAllAccount(String date) throws Exception {

        List<JSONObject> accountIds = adUnitReportService.getAllAccounts();
        if (Check.isNull(accountIds)) {
            return;
        }
        Map<Long, String> accessMap = new HashMap<>();

        List<JSONObject> agentTokens = tokenService.getAgentTokens();
        if (Check.isNull(agentTokens)) {
            return;
        }
        for (int i = 0; i < agentTokens.size(); i++) {
            JSONObject jsonObject = agentTokens.get(i);
            Long agentId = jsonObject.getLong("agentId");
            String accessToken = jsonObject.getString("accessToken");
            accessMap.put(agentId, accessToken);
        }
        String endDate = DateUtils.getNowDate("yyyy-MM-dd");
        String startDate = DateUtils.getAnotherDay("yyyy-MM-dd", endDate, -2);
        for (int i = 0; i < accountIds.size(); i++) {
            JSONObject jsonObject = accountIds.get(i);
            Long agentId = jsonObject.getLong("agentId");
            Long accountId = jsonObject.getLong("accountId");
            String token = accessMap.get(agentId);
            dailyFundExecutorService.submit(
                    new Runnable() {
                        @Override
                        public void run() {
                            adUnitReportService.getKuaishouDailyFlows(accountId, token, startDate, endDate, 1);
                        }
                    });


        }
    }

    @ApiOperation(value = "获取快手当天-广告组信息、组时报", notes = "更新快手当天-广告组信息、组时报")
    @GetMapping(value = "/getKuaishouAdUnitRealTime")
    public void getKuaishouVideoByAccountId(@RequestParam(name = "accountId") long accountId) {
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String token = tokenService.getByAccountId(accountId);
        //更新 组
        adUnitReportService.checkGroupInfo(accountId, token, nowDate, nowDate, 1);
        //更新 组时报
//        unitReportHourlyService.getUnitReportHour(accountId, token, nowDate, nowDate, 1);
    }


    @Autowired
    private IKuaishouUnitReportDailyService unitReportDailyService1;
    @Autowired
    private IKuaishouMaterialVideoReportDailyService videoReportDailyService;

    @Autowired
    private IKuaishouCampaignReportDailyService campaignReportDailyService;
    @Autowired
    private AdUnitReportServiceImpl reportService;


    @Autowired
    private IKuaishouUnitReportDailyService unitReportDailyService2;

    @Autowired
    private IKuaishouAccountReportHourlyService kuaishouAccountReportHourlyService;

    @GetMapping(value = "/testOne")
    public void testOne(@RequestParam(name = "accountId") long accountId) throws Exception {
        String token = tokenService.getByAccountId(accountId);
//        List<String> list = new ArrayList<>();
//        list.add("2023-11-10");
//        list.add("2023-08-26");
//        list.add("2023-08-27");
//        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
//        String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -4);
//        for (String s : list) {
//            unitReportDailyService2.getUnitReportDaily(accountId, token, s, s, 1);
//        adUnitReportService.getAgentReport();
        kuaishouAccountReportHourlyService
                .getAdvertiserReportHour(accountId, token, "2024-07-11", "2024-07-11", 1);
//        videoListService.getVideoList(accountId, token, null, null, 1);
//            campaignReportDailyService.getCampaignReportDaily(accountId, token, s, s, 1);
//        }

//        videoReportDailyService.getImageReportDaily(accountId, token, date, nowDate, 1);

//        videoListService.getVideosByAccountId(accountId, "2023-11-01", "2023-11-09");


    }

    @GetMapping(value = "/testAllDate")
    public void testAllDate() throws Exception {

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");

        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);

        List<Long> accountIds = tokenService.getAllAccountIds();
        for (Long accountId : accountIds) {
            String token = tokenService.getByAccountId(accountId);
            dailyExecutorService.submit(
                    new Runnable() {
                        @SneakyThrows
                        @Override
                        public void run() {
                            Thread.sleep(500);
                            kuaishouAccountReportDailyService.getAdvertiserReportDaily(accountId, token, "2024-02-16", "2024-02-18", 1);

                        }
                    });
        }
    }

    @GetMapping(value = "/testAllHour")
    public void testAllHour() throws Exception {
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterda = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);

        //获取 两个日期之间的所有 日期
        List<String> dates = DateUtils.getAllDatesOfTwoTimes("2024-02-16", "2024-02-18");

        List<Long> accountIds = tokenService.getAllAccountIds();

//        for (String date : dates) {
        for (Long accountId : accountIds) {
            String token = tokenService.getByAccountId(accountId);
            hourlyExecutorService.submit(
                    new Runnable() {
                        @SneakyThrows
                        @Override
                        public void run() {
                            Thread.sleep(500);
                            kuaishouAccountReportHourlyService
                                    .getAdvertiserReportHour(accountId, token, "2024-07-11", "2024-07-11", 1);
                        }
                    });
//            }
        }
    }


    /**
     * 获取原生效果数据整体概览接口（数据概览、分日转化数据分布、转化漏斗）
     *
     * @throws Exception
     */
    @GetMapping(value = "effectNativeChartReport")
    public void effectNativeChartReport(Long accountId) throws Exception {
        String token = tokenService.getByAccountId(Long.valueOf(accountId));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String beforeYesterday = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);

        kuaishouAccountReportDailyService.effectNativeChartReport(accountId, token, beforeYesterday, beforeYesterday);
    }

    @GetMapping(value = "getLiveInfo")
    public void getLiveInfo(Long accountId) throws Exception {
        String token = tokenService.getByAccountId(Long.valueOf(accountId));
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterday = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);

        kuaishouAccountReportDailyService.liveUserReport(accountId, token, yesterday, yesterday);

        kuaishouAccountReportDailyService.liveComponentReport(accountId, token, yesterday, yesterday);
    }

    @Autowired
    private CompassBindAdvertisersService compassBindAdvertisersService;

    @GetMapping(value = "testD")
    public void testD() throws Exception {
        List<Long> list = compassBindAdvertisersService.getAccountList();

        for (int i = 0; i < list.size(); i++) {
            Long accountId = list.get(i);
            CompassBindAdvertisers advertiser = compassBindAdvertisersService.getByAccountId(String.valueOf(accountId));
            String corporationName = advertiser.getCorporationName();
            String token;
            if (!Check.isNull(corporationName) && corporationName.contains("骄阳")) {
                //723 骄阳token
                token = tokenService.getAgentTokenByAgentId(723L);
            } else {
                //汇创
                token = tokenService.getAgentTokenByAgentId(14L);
            }
            dailyExecutorService.submit(
                    new Runnable() {
                        @Override
                        public void run() {
                            compassBindAdvertisersService.getCompassAdvertisersCampaignReportDaily(
                                    Long.valueOf(accountId), token, "2025-01-14", "2025-01-15", 1);
                        }
                    });

        }


    }


}