package cn.com.ctop.job.kuaishou.data.entity;

import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.alibaba.fastjson.JSONObject;
import lombok.Data;

/**
 * 代理商-广告主列对象 kuaishou_agent_account_list
 *
 * @author ruoyi
 * @date 2024-07-25
 */
@Data
public class KuaishouAgentAccountList {

    /**
     * 主键ID
     */
    private Long id;

    /**
     * 代理商ID
     */
    private Long agentId;
    /**
     * 快手ID
     */
    private Long userId;

    /**
     * 广告主ID
     */
    private Long accountId;

    /**
     * 快手昵称
     */
    private String accountName;

    /**
     * 账户类型
     */
    private String ucType;

    /**
     * 付款类型
     */
    private String paymentType;

    /**
     * 创建时间
     */
    private String createTime;

    private String statDate;

    /**
     * 现金余额
     */
    private Long balance;

    /**
     * 信用账户余额
     */
    private Long creditBalance;

    /**
     * 预留账户余额
     */
    private Long extendedBalance;

    /**
     * 后返余额
     */
    private Long rebate;

    /**
     * 前返余额
     */
    private Long preRebate;

    /**
     * 框返余额
     */
    private Long contractRebate;

    /**
     * 激励余额
     */
    private Long directRebate;

    /**
     * 总余额
     */
    private Long totalBalance;

    /**
     * 账户最低余额
     */
    private Long loLimit;

    /**
     * 单次转账金额
     */
    private Long singleOut;

    /**
     * 自动转账状态 1-ture,0-false
     */
    private String autoOut;

    /**
     * 余额不足提醒 1-ture,0-false
     */
    private String balanceOut;

    /**
     * 产品名称
     */
    private String productName;

    /**
     * 首日消耗日期
     */
    private String firstCostDay;

    /**
     * 一级行业
     */
    private String industry;

    /**
     * 二级行业
     */
    private String secondIndustry;

    /**
     * $column.columnComment
     */
    private Long kuaibiBalance;

    /**
     * $column.columnComment
     */
    private Long pushBalance;

    /**
     * 1-ture,0-false
     */
    private String recharged;

    /**
     * 企业名称
     */
    private String corporationName;

    /**
     * 审核状态	1-审核中，2-审核通过，3-审核拒绝，0-待提交
     */
    private Integer reviewStatus;
    /**
     * 审核详情
     */
    private String reviewDetail;

    /**
     * 冻结状态
     */
    private Integer forzenStatus;

    /**
     * 转账状态  1-ture,0-false
     */
    private String transferAccountStatus;

    private Long appId;

    /**
     * 创建来源
     */
    private Integer createSource;

    /**
     * 账号认证状态	0-无状态，1-待认证，2-认证中，3-认证通过，4-认证失败，5-认证失效
     */
    private Integer authenticationStatus;

    /**
     * 账号认证状态详情
     */
    private String authenticationDetail;

    /**
     * 合同签约状态	0-无效，1-签约中，2-已签约，3-已终止，4-已过期，5-已撤销，6-已拒签，7-已删除，8-提前终止
     */
    private Integer contractReviewStatus;

    /**
     * 合同签约状态详情
     */
    private String contractReviewDetail;

    /**
     * 用户审核状态	0-未送审，1-待审核，2-审核通过，3-审核拒绝，4-封禁
     */
    private Integer userReviewStatus;

    /**
     * 用户审核状态详情
     */
    private String userReviewDetail;

    /**
     * 是否为复制账户 1-ture,0-false
     */
    private String copyAccount;

    public KuaishouAgentAccountList() {
    }

    public KuaishouAgentAccountList(JSONObject json, Long agentId) {
        this.appId = json.getLong("app_id");
        this.agentId = agentId;
        this.userId = json.getLong("user_id");
        this.accountId = json.getLong("account_id");
        this.accountName = json.getString("account_name");
        this.ucType = json.getString("uc_type");
        this.paymentType = json.getString("payment_type");
        this.balance = json.getLong("balance");
        this.createTime = DateUtils.tempToDate(json.getLong("create_time"), "yyyy-MM-dd HH:mm:ss");
        this.statDate = DateUtils.tempToDate(json.getLong("create_time"), "yyyy-MM-dd");
        this.creditBalance = json.getLong("credit_balance");
        this.extendedBalance = json.getLong("extended_balance");
        this.rebate = json.getLong("rebate");
        this.preRebate = json.getLong("pre_rebate");
        this.contractRebate = json.getLong("contract_rebate");
        this.directRebate = json.getLong("direct_rebate");
        this.totalBalance = json.getLong("total_balance");
        this.loLimit = json.getLong("lo_limit");
        this.singleOut = json.getLong("single_out");
        Boolean auto_out = json.getBoolean("auto_out");
        if (!Check.isNull(auto_out)) {
            this.autoOut = auto_out ? "1" : "0";
        }
        Boolean balance_out = json.getBoolean("balance_out");
        if (!Check.isNull(balance_out)) {
            this.balanceOut = balance_out ? "1" : "0";
        }
        this.productName = json.getString("product_name");
        this.firstCostDay = json.getString("first_cost_day");
        this.industry = json.getString("industry");
        this.secondIndustry = json.getString("second_industry");
        this.kuaibiBalance = json.getLong("kuaibi_balance");
        this.pushBalance = json.getLong("push_balance");
        Boolean recharged = json.getBoolean("recharged");
        if (!Check.isNull(recharged)) {
            this.recharged = recharged ? "1" : "0";
        }
        this.corporationName = json.getString("corporation_name");
        this.reviewStatus = json.getInteger("review_status");
        this.reviewDetail = json.getString("review_detail");
        this.forzenStatus = json.getInteger("forzen_status");
        Boolean transfer_account_status = json.getBoolean("transfer_account_status");
        if (!Check.isNull(transfer_account_status)) {
            this.transferAccountStatus = transfer_account_status ? "1" : "0";
        }
        this.createSource = json.getInteger("create_source");
        JSONObject info = json.getJSONObject("child_review_status_info");
        if (!Check.isNull(info)) {
            JSONObject contractReviewInfo = info.getJSONObject("contract_review_info");
            if (!Check.isNull(contractReviewInfo)) {
                this.contractReviewStatus = contractReviewInfo.getInteger("contract_review_status");
                this.contractReviewDetail = contractReviewInfo.getString("contract_review_detail");
            }
            JSONObject authenticationReviewInfo = info.getJSONObject("authentication_review_info");
            if (!Check.isNull(authenticationReviewInfo)) {
                this.authenticationStatus = authenticationReviewInfo.getInteger("authentication_status");
                this.authenticationDetail = authenticationReviewInfo.getString("authentication_detail");
            }
            JSONObject userReviewInfo = info.getJSONObject("user_review_info");
            if (!Check.isNull(userReviewInfo)) {
                this.userReviewStatus = userReviewInfo.getInteger("review_status");
                this.userReviewDetail = userReviewInfo.getString("review_detail");
            }
        }
        Boolean copy_account = json.getBoolean("copy_account");
        if (!Check.isNull(copy_account)) {
            this.copyAccount = copy_account ? "1" : "0";
        }
    }
}
