package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.entity.Project;
import cn.com.ctop.job.kuaishou.data.entity.UserAllocation;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouReportHourlyAccountMapper;
import cn.com.ctop.job.kuaishou.data.service.IAccountWarningService;
import cn.com.ctop.job.kuaishou.data.service.IUserAllocationService;
import cn.com.ctop.job.kuaishou.data.service.IWexinConfigService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.weixin.CorpWexinUtils;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

@Service
@Slf4j
public class AccountWarningServiceImpl implements IAccountWarningService {
    @Autowired
    private KuaishouReportHourlyAccountMapper hourlyAccountMapper;
    @Autowired
    private IUserAllocationService userAllocationService;
    @Autowired
    private CorpWexinUtils corpWexinUtils;
    @Resource
    private IWexinConfigService wexinService;


    /**
     * 快手账户 出价预警
     *
     * @param accountId
     */
    @Override
    public void kuaiShouAccountWarning(Long accountId, Integer today, Integer yesterday) {
        try {
            Integer maxHour = hourlyAccountMapper.selectMaxHour(accountId, today);
            if (Check.isNull(maxHour)) {
                return;
            }
            UserAllocation userAllocation = userAllocationService.getByAccountId(accountId);
            if (Check.isNull(userAllocation)) {
                return;
            }
            if (userAllocation.getAccountStatus() == 1) {
                return;
            }
            Project project = userAllocationService.getProjectEntity(userAllocation.getProjectId());
            BigDecimal warningAmount = userAllocation.getWarningAmount();
            BigDecimal warningProportion = userAllocation.getWarningProportion();
            BigDecimal nowCost = hourlyAccountMapper.getCost(accountId, maxHour, today);
            if (maxHour != 0) {
                if (!Check.isNull(nowCost) && nowCost.compareTo(new BigDecimal(0)) != 0) {
                    BigDecimal lastHourCost = hourlyAccountMapper.getCost(accountId, maxHour - 1, today);
                    if (!Check.isNull(lastHourCost) && lastHourCost.compareTo(new BigDecimal(0)) != 0) {
                        BigDecimal subtract = nowCost.subtract(lastHourCost);
                        BigDecimal divide = nowCost.divide(lastHourCost, 2, RoundingMode.HALF_UP);
                        if (subtract.compareTo(warningAmount) > -1 && divide.compareTo(warningProportion) > -1) {
                            sendMessage(accountId, userAllocation.getAuthName(), maxHour, nowCost, userAllocation.getProjectId(), project.getProjectName(), userAllocation.getUserName());
                        }
                    }
                }
            }
            if (!Check.isNull(nowCost) && nowCost.compareTo(new BigDecimal(0)) != 0) {
                BigDecimal yesterdayHourCost = hourlyAccountMapper.getCost(accountId, maxHour, yesterday);
                if (!Check.isNull(yesterdayHourCost) && yesterdayHourCost.compareTo(new BigDecimal(0)) != 0) {
                    BigDecimal subtract = nowCost.subtract(yesterdayHourCost);
                    BigDecimal divide = nowCost.divide(yesterdayHourCost, 2, RoundingMode.HALF_UP);
                    if (subtract.compareTo(warningAmount) > -1 && divide.compareTo(warningProportion) > -1) {
                        sendYesterdayMessage(accountId, userAllocation.getAuthName(), maxHour, nowCost, userAllocation.getProjectId(), project.getProjectName(), userAllocation.getUserName());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private void sendYesterdayMessage(Long accountId, String authName, Integer maxHour, BigDecimal nowCost, Long projectId, String projectName, String userName) {
        StringBuilder text = new StringBuilder();
        text.append("较昨日同时段消耗预警").append("<br/>")
                .append("您的项目：" + projectName + "下的，").append("<br/>")
                .append("账号：").append(accountId + "，").append("授权名称为：" + authName).append("<br/>")
                .append("账户所属人:" + userName).append("<br/>")
                .append("相较于昨日 " + maxHour + "时").append("<br/>")
                .append("出现消耗暴涨，").append("消耗金额为：" + nowCost).append("<br/>")
                .append("请您及时查看！");
        List<JSONObject> wexinIds = wexinService.selectUsersByAccountId(accountId);
        corpWexinUtils.sendMessage(wexinIds, text.toString());
    }

    private void sendMessage(Long accountId, String authName, Integer hour, BigDecimal amount, Long projectId, String projectName, String userName) {
        StringBuilder text = new StringBuilder();
        text.append("较上时段消耗预警").append("<br/>")
                .append("您的项目：" + projectName + " 下的").append("<br/>")
                .append("账号：").append(accountId + "，").append("授权名称为：" + authName).append("<br/>")
                .append("账户所属人:" + userName).append("<br/>")
                .append(hour + " 时").append("相较于 " + (hour - 1) + " 时").append("<br/>")
                .append("出现消耗暴涨，").append("消耗金额为：" + amount).append("<br/>")
                .append("请您及时查看！");
        List<JSONObject> wexinIds = wexinService.selectUsersByAccountId(accountId);
        corpWexinUtils.sendMessage(wexinIds, text.toString());
    }

}
