package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.constant.KuaishouConstant;
import cn.com.ctop.job.kuaishou.data.entity.KuaiShouDailyAgent;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAccountAdSceneReportDaily;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAdUnitList;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAdvertiserFundDailyFlows;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAdvertiserFundGet;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAgentAccountList;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAudienceReportDaily;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouCampaignList;
import cn.com.ctop.job.kuaishou.data.entity.OauthAgentToken;
import cn.com.ctop.job.kuaishou.data.mapper.AdUnitReportMapper;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouAccountAdSceneReportDailyMapper;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouAdvertiserFundDailyFlowsMapper;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouAudienceReportDailyMapper;
import cn.com.ctop.job.kuaishou.data.mapper.OauthAgentTokenMapper;
import cn.com.ctop.job.kuaishou.data.mapper.OauthTokenMapper;
import cn.com.ctop.job.kuaishou.data.service.IAdUnitReportService;
import cn.com.ctop.job.kuaishou.data.service.IWarningOperationService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.xxl.job.core.context.XxlJobHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 快手 定时任务
 *
 * @author zian Y 2022-01-05
 * @version V1.0
 */
@Slf4j
@Service
public class AdUnitReportServiceImpl implements IAdUnitReportService {

    @Value("${api.kuaishou.postUrl}")
    private String postUrl;

    @Resource
    private OauthTokenMapper tokenMapper;

    @Resource
    private OauthAgentTokenMapper agentTokenMapper;

    @Resource
    private AdUnitReportMapper adUnitReportMapper;

    @Resource
    private KuaishouAccountAdSceneReportDailyMapper accountAdSceneReportDailyMapper;

    @Resource
    private KuaishouAudienceReportDailyMapper audienceReportDailyMapper;

    @Resource
    private KuaishouAdvertiserFundDailyFlowsMapper fundDailyFlowsMapper;


    @Resource
    private IWarningOperationService warningOperationService;


    /**
     * 获取广告组信息
     *
     * @param advertiserId
     * @param accessToken
     * @param startDate
     * @param endDate
     * @param page
     */
    @Override
    public void getAdUnitReport(Long advertiserId, Long unitId, String accessToken, String startDate, String endDate, int page) {
        JSONObject param = new JSONObject();
        param.put("advertiser_id", advertiserId);
        param.put("page_size", 500);
        param.put("page", page);
        if (startDate != null && endDate != null) {
            param.put("start_date", startDate);
            param.put("end_date", endDate);
        }
        if (unitId != null) {
            param.put("unit_id", unitId);
        }
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.kuaiShouhttpPostRequest(postUrl + KuaishouConstant.AD_UNIT_LIST, param.toJSONString(), headers);
        JSONObject resultJson = JSONObject.parseObject(result);

        if (Check.isNull(resultJson)) {
            log.info("accountId:=={}=====广告组数据信息====暂无====》》》》", advertiserId);
            return;
        }
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手广告组数据异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.info("快手广告组数据为空=》accountId:{}", advertiserId);
            return;
        }

        List<KuaishouAdUnitList> ksAdUnitList = new ArrayList<>();
        for (Object detail : details) {
            JSONObject unitDetail = (JSONObject) detail;
            unitDetail.put("advertiser_id", advertiserId);
            KuaishouAdUnitList ksAdunit = KuaishouAdUnitList.getAdUnitListInfo(unitDetail);

            ksAdUnitList.add(ksAdunit);
        }

        if (!Check.isNull(ksAdUnitList)) {
            adUnitReportMapper.replaceBatchKsAdUnit(ksAdUnitList);
            log.info("组数据添加成功，accountId：{},条数：{}", advertiserId, ksAdUnitList.size());
        }

        //是否有下一页
        Boolean toGet = true;
        if (details.size() < page) {
            toGet = false;
        }
        if (toGet) {
            getAdUnitReport(advertiserId, unitId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("快手广告组数据获取完成：accountId：{}", advertiserId);
        }

    }

    @Override
    public void checkGroupInfo(Long accountId, String accessToken, String startDate, String endDate, int page) {
        JSONObject param = new JSONObject();
        param.put("advertiser_id", accountId);
        param.put("page_size", 500);
        param.put("page", page);
        if (startDate != null && endDate != null) {
            param.put("start_date", startDate);
            param.put("end_date", endDate);
        }
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.kuaiShouhttpPostRequest(postUrl + KuaishouConstant.AD_UNIT_LIST, param.toJSONString(), headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            log.error("accountId:{} =》=》=》AdUnitReportServiceImpl.checkGroupInfo返回广告组数据为空", accountId);
            return;
        }
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("accountId:{} =》=》=》AdUnitReportServiceImpl.checkGroupInfo\n返回广告组数据异常,message:{}", accountId, message);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            return;
        }

        checkAndInsertGroup(accountId, details);
        //是否有下一页
        Boolean toGet = true;
        if (details.size() < page) {
            toGet = false;
        }
        if (toGet) {
            checkGroupInfo(accountId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info(" =》AdUnitReportServiceImpl.checkGroupInfo =》快手广告组数据获取完成：accountId：{}", accountId);
        }

    }

    static ExecutorService executorBidTypeService = Executors.newFixedThreadPool(20);
    static ExecutorService executorMaxBidService = Executors.newFixedThreadPool(5);

    private void checkAndInsertGroup(Long accountId, JSONArray details) {
        JSONObject project = adUnitReportMapper.getBidTypeAndMaxBid(accountId);
        if (Check.isNull(project) || Check.isNull(details)) {
            return;
        }
        Map<String, String> textMap = new HashMap<>();
        List<JSONObject> lists = tokenMapper.getKuaishouOcpxTypeList();
        for (JSONObject da : lists) {
            textMap.put(da.getString("ocpxActionType"), da.getString("ocpxActionName"));
        }
        // 最高出价
        Long maxBid = project.getLong("maxBid");

        //优化目标集
        List<Integer> bidList = null;
        String bidTypeStr = project.getString("bidType");
        if (!Check.isNull(bidTypeStr)) {
            bidList = JSONArray.parseArray(bidTypeStr, Integer.class);
        }

        //转化目标map
        Map<Integer, String> ocpxMap = new HashMap<>();
        String ocpxTypeStr = project.getString("ocpxActionType");
        if (!Check.isNull(ocpxTypeStr)) {
            JSONArray ocpxArr = JSONArray.parseArray(ocpxTypeStr);
            StringBuffer nameStr = new StringBuffer();
            for (int i = 0; i < ocpxArr.size(); i++) {
                JSONObject obj = ocpxArr.getJSONObject(i);
                ocpxMap.put(obj.getInteger("ocpxCode"), obj.getString("maxValue"));
                nameStr.append(textMap.get(obj.getString("ocpxCode"))).append(" ");
            }
            if (ocpxArr.size() > 0) {
                ocpxMap.put(199999, nameStr.toString());
            }
        }

        //深度转化目标map
        Map<Integer, String> deepMap = new HashMap<>();
        String deepTypeStr = project.getString("deepConversionType");
        if (!Check.isNull(deepTypeStr)) {
            JSONArray deepArr = JSONArray.parseArray(deepTypeStr);
            StringBuffer nameStr = new StringBuffer();
            for (int i = 0; i < deepArr.size(); i++) {
                JSONObject obj = deepArr.getJSONObject(i);
                deepMap.put(obj.getInteger("deepCode"), obj.getString("deepValue"));
                nameStr.append(textMap.get(obj.getString("deepCode"))).append(" ");
            }
            if (deepArr.size() > 0) {
                deepMap.put(199999, nameStr.toString());
            }
        }
        List<KuaishouAdUnitList> ksAdUnitList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject detail = JSONObject.parseObject(details.get(i).toString());
            if (!Check.isNull(detail)) {
                //优化目标
                Integer bidType = detail.getInteger("bid_type");
                //优化出价（只有单击时，才有值）
                Long bid = detail.getLong("bid");
                //转化目标
                Integer ocpxActionType = detail.getInteger("ocpx_action_type");
                //转化目标出价
                Long cpaBid = detail.getLong("cpa_bid");
                //深度转化目标
                Integer deepConversionType = detail.getInteger("deep_conversion_type");
                //深度转化目标出价
                Long deepConversionBid = detail.getLong("deep_conversion_bid");
                //单日预算
                Long dayBudget = detail.getLong("day_budget");
                //组ID
                Long unitId = detail.getLong("unit_id");
                //组名称
                String unitName = detail.getString("unit_name");
                //状态
                Integer status = detail.getInteger("status");
                //投放状态
                Integer putStatus = detail.getInteger("put_status");
                if (putStatus == 1 && status != 15) { // 组状态为 投放中的 and 广告组状态为非暂停
                    Boolean typeFalg = true;
                    Boolean bidFalg = true;

                    StringBuffer msgType = new StringBuffer();
                    StringBuffer msgBid = new StringBuffer();
                    //优化目标
                    if (!bidList.isEmpty()) {
                        if (!bidList.contains(bidType)) {
                            //项目不包含该优化目标，则关停
                            typeFalg = false;
                            msgType.append("已设优化：").append(bidTypeStr).append("<br/>").append("，目标优化：").append(bidType);
                        } else if (bidType == 2 && bid > maxBid) {
                            //若优化目标为2点击数，并且广告组出价大于项目设置最大出价，则关停
                            bidFalg = false;
                            msgBid.append("目标优化为点击数，且组出价：").append(bid).append("大于项目最大出价：").append(maxBid);
                        }
                    }
                    //转化目标
                    if (!Check.isNull(ocpxActionType) && ocpxActionType != 0) {
                        String ocpxStr = ocpxActionType.toString();
                        String octypeName = Check.isNull(textMap.get(ocpxStr)) ? ocpxStr : textMap.get(ocpxStr);
                        //！=0是设置过转化目标
                        if (!Check.isNullMap(ocpxMap)) {
                            String ocpxBid = ocpxMap.get(ocpxActionType);
                            if (Check.isNull(ocpxBid)) {
                                if (ocpxActionType == 2) {
                                    if (dayBudget == 0 || dayBudget > 500 * 1000) {
                                        // 转化目标为2行为数，并且预算大于500 或者不限，则关停
                                        msgType.append("目标转化是行为数，");
                                        if (dayBudget == 0) {
                                            msgType.append("且组单日预算设置为不限！");
                                        } else {
                                            msgType.append("且组单日预算：").append(dayBudget / 1000).append("大于500！");
                                        }
                                    }
                                } else {
                                    //通过key未获取值，说明不包含该目标，进行关停
                                    typeFalg = false;
                                    msgType.append("已设转化：").append(ocpxMap.get(199999)).append("<br/>").append("目标转化：").append(octypeName);
                                }
                            } else {
                                if (cpaBid > Long.valueOf(ocpxBid)) {
                                    //组转化出价 大于 项目设置最大转化出价，则关停
                                    bidFalg = false;
                                    msgBid.append("[").append(octypeName).append("]最高出价：").append(Long.valueOf(ocpxBid) / 1000).append("<br/>");
                                    msgBid.append("目标出价：").append(cpaBid / 1000);
                                }
                            }
                        } else if (ocpxActionType == 2 && (dayBudget == 0 || dayBudget > 500 * 1000)) {
                            // 转化目标为2行为数，并且预算大于500 或者不限，则关停
                            typeFalg = false;
                            msgType.append("目标转化是行为数，");
                            if (dayBudget == 0) {
                                msgType.append("且组单日预算设置为不限！");
                            } else {
                                msgType.append("且组单日预算：").append(dayBudget / 1000).append("大于500！");
                            }
                        }
                    }

                    //深度转化目标
                    if (!Check.isNull(deepConversionType) && deepConversionType != 0 && !deepMap.isEmpty()) {
                        String deepStr = deepConversionType.toString();
                        String depTypeName = Check.isNull(textMap.get(deepStr)) ? deepStr : textMap.get(deepStr);
                        String deepBid = deepMap.get(deepConversionType);
                        if (Check.isNull(deepBid)) {
                            //通过key未获取值，说明不包含该目标，进行关停
                            typeFalg = false;
                            msgType.append("已设深度：").append(deepMap.get(199999)).append("<br/>").append("目标深度：").append(depTypeName);
                        } else {
                            if (deepConversionBid > Long.valueOf(deepBid)) {
                                //组深度转化出价 大于 项目设置最大转化出价，则关停
                                bidFalg = false;
                                msgBid.append("[").append(depTypeName).append("]最高深度出价：").append(Long.valueOf(deepBid) / 1000).append("<br/>");
                                msgBid.append("目标深度出价：").append(deepConversionBid / 1000);
                            }
                        }
                    }

                    if (!typeFalg) { // 出价方式设置非法
                        executorBidTypeService.submit(new Runnable() {
                            @Override
                            public void run() {
                                warningOperationService.kuaiShouWarningOperation(project.getLong("projectId"), project.getString("projectName"), accountId, unitId, unitName, msgType.toString());
                            }
                        });
                    }
                    if (!bidFalg) {
                        executorMaxBidService.submit(new Runnable() {
                            @Override
                            public void run() {
                                warningOperationService.kuaiShouBidWarningOperation(project.getLong("projectId"), project.getString("projectName"), accountId, unitId, unitName, msgBid.toString());
                            }
                        });
                    }
                }

                detail.put("advertiser_id", accountId);
                KuaishouAdUnitList ksAdunit = KuaishouAdUnitList.getAdUnitListInfo(detail);
                ksAdUnitList.add(ksAdunit);

            }
        }
        if (!Check.isNull(ksAdUnitList)) {
            adUnitReportMapper.replaceBatchKsAdUnit(ksAdUnitList);
            log.info("------【accountId：{} 】添加广告组DATA成功，执行数量：{}", accountId, ksAdUnitList.size());
        }
    }


    /**
     * 修改广告组状态
     * putStatus 1-投放、2-暂停、3-删除，传其他数字非法
     */
    @Override
    public void updateUnitStatus(String token, Long acccountId, Long unitId, int putStatus) {
        try {
            String url = postUrl + KuaishouConstant.UPDATE_UNIT_STATUS;
            Map<String, String> header = new HashMap<>();
            header.put("Access-Token", token);
            header.put("Content-Type", "application/json");
            JSONObject params = new JSONObject();
            params.put("advertiser_id", acccountId);
            params.put("unit_id", unitId);
            params.put("put_status", putStatus);
            String result = HttpUtils.httpPostRequest(url, params, header);
            JSONObject resultJson = JSONObject.parseObject(result);
            if (!Check.isNull(resultJson)) {
                Integer code = resultJson.getInteger("code");
                if (code == 0) {
                    if (putStatus == 3) {
                        Map<String, Object> deleteMap = new HashMap<>();
                        deleteMap.put("accountId", acccountId);
                        deleteMap.put("unitId", unitId);
                        adUnitReportMapper.deleteByMap(deleteMap);
                    } else {
                        getAdUnitReport(acccountId, unitId, token, null, null, 1);
                    }
                } else {
                    log.error("=》=》=》AdUnitReportServiceImpl.updateUnitStatus修改广告组状态失败，advertiserId：{}，unitId：{}，返回信息：{}", acccountId, unitId, resultJson);
                }
            } else {
                log.error("修改广告组状态返回结果为空，advertiserId：{}，unitId：{}", acccountId, unitId);
            }
        } catch (Exception e) {
            log.error("修改广告组状态异常，advertiserId：{}，unitId：{}", acccountId, unitId);
            e.printStackTrace();

        }
    }

    /**
     * 快手广告计划信息
     *
     * @param advertiserId
     * @param accessToken
     * @param startDate
     * @param endDate
     * @param page
     */
    @Override
    public void getAdPlanReport(Long advertiserId, String accessToken, String startDate, String endDate, int page) {

        log.info("accountId:=={}=====获取广告计划数据信息====page--{}===》》》》", advertiserId, page);
        Integer page_size = 500;

        JSONObject param = new JSONObject();
        param.put("advertiser_id", advertiserId);
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("page_size", 500);
        param.put("page", page);

        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.httpPostRequest(postUrl + KuaishouConstant.CAMPAIGN_LIST, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            XxlJobHelper.log("获取广告计划接口异常,advertiserId:{}", advertiserId);
            return;
        }
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            XxlJobHelper.log("获取广告计划返回结果异常,advertiserId:{},异常信息:{}", advertiserId, resultJson);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (Check.isNull(details)) {
            log.info("快手广告计划数据为空=》accountId:{}", advertiserId);
            return;
        }

        List<KuaishouCampaignList> kscampaignList = new ArrayList<>();
        for (Object detail : details) {
            JSONObject unitDetail = (JSONObject) detail;
            unitDetail.put("advertiser_id", advertiserId);
            KuaishouCampaignList ksAdunit = KuaishouCampaignList.getKuaishouCampaignListInfo(unitDetail);
            kscampaignList.add(ksAdunit);
        }
        if (!Check.isNull(kscampaignList)) {
            adUnitReportMapper.replaceBatchKsCampaign(kscampaignList);
        }

        //是否有下一页
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        if (toGet) {
            getAdPlanReport(advertiserId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("快手广告计划数据获取完成：accountId：{}", advertiserId);
        }

    }

    /**
     * 快手-代理商数据
     */
    @Override
    public void getAgentReport() {
        // 前一天 时间
        String date = DateUtils.getAnotherDay("yyyy-MM-dd", DateUtils.getNowDate("yyyy-MM-dd"), -2);
        String date1 = DateUtils.getAnotherDay("yyyy-MM-dd", DateUtils.getNowDate("yyyy-MM-dd"), -1);
        List<Long> appList = new ArrayList<>();
        appList.add(27L);
        appList.add(165893351L);
        for (int i = 0; i < appList.size(); i++) {
            Long appId = appList.get(i);
            log.info("获取代理商数据appId:{}", appId);
            OauthAgentToken agentToken = agentTokenMapper.getTokenByAppId(appId);
            log.info("获取代理商数据授权信息：{}", agentToken);
            getAgentReport(agentToken.getAccessToken(), agentToken.getAgentId(), date, date1, 1);
        }

    }


    /**
     * 获取代理商数据
     *
     * @param token
     * @param agentId
     * @param startDate
     * @param endDate
     * @param page
     */
    private void getAgentReport(String token, Long agentId, String startDate, String endDate, int page) {
        try {
            Thread.sleep(500L);
            Integer page_size = 1000;
            log.info("=====获取代理商数据信息====page--{}===《《《《《", page);
            Map<String, String> headers = new HashMap<String, String>();
            headers.put("Content-Type", "application/json");
            headers.put("Access-Token", token);
            JSONObject param = new JSONObject();
            param.put("start_date", startDate);
            param.put("end_date", endDate);
            param.put("agent_id", agentId);
            param.put("page_size", page_size);
            param.put("page", page);

            String result = HttpUtils.httpPostRequest(postUrl + KuaishouConstant.ANGENT_REPORT, param, headers);
            JSONObject resultJson = JSONObject.parseObject(result);
            if (Check.isNull(resultJson)) {
                log.info("代理商数据返回为空");
                return;
            }
            Integer code = resultJson.getInteger("code");
            if (code != 0) {
                log.error("代理商报表返回code错误，返回：{}", resultJson);
                return;
            }
            JSONObject data = resultJson.getJSONObject("data");
            if (Check.isNull(data)) {
                log.error("代理商报表返回data节点为空");
                return;
            }
            JSONArray details = data.getJSONArray("details");
//            System.out.println(details);
            if (Check.isNull(details)) {
                log.error("代理商报表返回details节点为空");
                return;

            }
            List<KuaiShouDailyAgent> agentList = new ArrayList<>();
            for (int i = 0; i < details.size(); i++) {
                JSONObject detailJson = details.getJSONObject(i);
                if (!Check.isNull(detailJson)) {
                    detailJson.put("agent_id", agentId);
                    KuaiShouDailyAgent agent = KuaiShouDailyAgent.getKuaishouAgentDailyInfo(detailJson);
                    agentList.add(agent);
                }
            }
            if (!Check.isNull(agentList)) {
                adUnitReportMapper.replaceBatchAgentReport(agentList);
            }
            //是否有下一页
            Boolean toGet = true;
            if (details.size() < page_size) {
                toGet = false;
            }
            if (toGet) {
                getAgentReport(token, agentId, startDate, endDate, page + 1);
            } else {
                log.info("快手代理商数据获取完成========");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    /**
     * @param advertiserId
     * @param accessToken
     * @param startDate
     * @param endDate
     * @param page
     * @param temporalGranularity 天粒度（DAILY）／小时粒度（HOURLY），默认支持天粒度数据
     * @param reportDims          "adScene"：按广告场景;"placementType"：按广告范围，快手/联盟;不传/传空/传空数组：不限
     * @description: 广告主数据
     * @return: void
     * @author: zianY
     * @time: 2022/1/6
     */
    @Override
    public void getKuaishouAccountAdSceneDaily(Long advertiserId, String accessToken, String startDate, String endDate, int page, String temporalGranularity, List<String> reportDims) {
        log.info("获取账户日报====广告场景====，账户id：{}，开始时间：{}，结束时间：{}，page--{}", advertiserId, startDate, endDate, page);
        Integer page_size = 2000;
        String url = postUrl + KuaishouConstant.ACCOUNT_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", temporalGranularity);
        param.put("report_dims", reportDims);
        param.put("page_size", page_size);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手广告主日报===按广告场景=====异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手日报===按广告场景=====返回详情为空，accountId：{}", advertiserId);
            return;
        }
        List<KuaishouAccountAdSceneReportDaily> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            returnJson.put("advertiser_id", advertiserId);
            KuaishouAccountAdSceneReportDaily reportDaily = KuaishouAccountAdSceneReportDaily.getKuaishouAccountAdsecneInfo(returnJson);
            addList.add(reportDaily);
        }
        if (!Check.isNull(addList)) {
            accountAdSceneReportDailyMapper.replaceBatchKsAccountAdScene(addList);
        }
        //是否有下一页
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        if (toGet) {
            getKuaishouAccountAdSceneDaily(advertiserId, accessToken, startDate, endDate, page + 1, temporalGranularity, reportDims);
        } else {
            log.info("账户日报数据====广告场景=====获取完成：accountId：{}", advertiserId);
        }

    }


    /**
     * 快手-人群报表
     *
     * @param advertiserId
     * @param accessToken
     * @param date
     * @param campaignIds  广告计划 ID 集
     * @param unitIds      广告组 ID 集
     * @param reportDims   人群报表类型
     * @param page
     */
    @Override
    public void crowdAnalysisReport(Long advertiserId, String accessToken, String date, List<Long> campaignIds, List<Long> unitIds, List<String> reportDims, Integer page) {
        log.info("获取人群报表===={}====，账户id：{}，开始时间：{}，结束时间：{}，page==={}", reportDims, advertiserId, date, date, page);
        try {
            Integer page_size = 2000;
            Map<String, String> headers = new HashMap<>();
            headers.put("Access-Token", accessToken);
            headers.put("Content-Type", "application/json");
            JSONObject requestJson = new JSONObject();
            requestJson.put("advertiser_id", advertiserId);
            requestJson.put("start_date", date);
            requestJson.put("end_date", date);
            requestJson.put("campaign_ids", campaignIds);
            requestJson.put("unit_ids", unitIds);
            requestJson.put("report_dims", reportDims);
            requestJson.put("page", page);
            requestJson.put("page_size", page_size);
            String result = HttpUtils.httpPostRequest(postUrl + KuaishouConstant.AUDIENCE_REPORT, requestJson, headers);
            JSONObject resultJson = JSONObject.parseObject(result);
            Integer code = resultJson.getInteger("code");
            if (null == code || code != 0) {
                log.error("获取快手======{}===人群报表=====异常:{},accountId:{}", reportDims, resultJson.getString("message"), advertiserId);
                return;
            }
            JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
            if (null == details || details.size() <= 0) {
                log.error("快手人群报表==={}=====返回详情为空，accountId：{}", reportDims, advertiserId);
                return;
            }
            List<KuaishouAudienceReportDaily> provinceList = new ArrayList<>();
            List<KuaishouAudienceReportDaily> cityList = new ArrayList<>();
            List<KuaishouAudienceReportDaily> ageList = new ArrayList<>();
            List<KuaishouAudienceReportDaily> genderList = new ArrayList<>();
            for (int i = 0; i < details.size(); i++) {
                JSONObject returnJson = details.getJSONObject(i);
                returnJson.put("advertiser_id", advertiserId);
                KuaishouAudienceReportDaily reportDaily = KuaishouAudienceReportDaily.getKuaishouAudienceInfo(returnJson);
                //省份
                if (reportDims.contains("province")) {
                    provinceList.add(reportDaily);
                }
                //城市
                if (reportDims.contains("city")) {
                    cityList.add(reportDaily);
                }
                //年龄
                if (reportDims.contains("ageSegment")) {
                    ageList.add(reportDaily);
                }
                //性别
                if (reportDims.contains("gender")) {
                    genderList.add(reportDaily);
                }
            }

            if (reportDims.contains("province")) {
                audienceReportDailyMapper.replaceBatchKsAudienceProvince(provinceList);
            }
            if (reportDims.contains("city")) {
                audienceReportDailyMapper.replaceBatchKsAudienceCity(cityList);
            }
            if (reportDims.contains("ageSegment")) {
                audienceReportDailyMapper.replaceBatchKsAudienceAge(ageList);
            }
            if (reportDims.contains("gender")) {
                audienceReportDailyMapper.replaceBatchKsAudienceGender(genderList);
            }
            //是否有下一页
            Boolean toGet = true;
            if (details.size() < page_size) {
                toGet = false;
            }
            if (toGet) {
                crowdAnalysisReport(advertiserId, accessToken, date, campaignIds, unitIds, reportDims, page + 1);
            } else {
                log.info("快手人群报表==={}==========获取完成：accountId：{}", reportDims, advertiserId);
            }
        } catch (Exception e) {
            log.info(String.valueOf(e));
        }

    }

    /**
     * 快手-账户流水
     *
     * @param advertiserId
     * @param accessToken
     * @param page
     */
    @Override
    public void getKuaishouDailyFlows(Long advertiserId, String accessToken, String startDate, String endDate, int page) {
        log.info("获取账户流水========，账户id：{}，page--{}", advertiserId, page);
        Integer page_size = 500;
        String url = postUrl + KuaishouConstant.FUND_DAILY_FLOWS;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("advertiser_id", advertiserId);
        param.put("page_size", page_size);
        param.put("page", page);
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手账户流水======异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手账户流水=====返回详情为空，accountId：{}", advertiserId);
            return;
        }
        List<KuaishouAdvertiserFundDailyFlows> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouAdvertiserFundDailyFlows reportDaily = KuaishouAdvertiserFundDailyFlows.getKuaishouFundDailyFlowsInfo(returnJson);
            addList.add(reportDaily);
        }
        if (!Check.isNull(addList)) {
            fundDailyFlowsMapper.replaceBatchKsFundDailyFlows(addList);
        }
        //是否有下一页
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        if (toGet) {
            getKuaishouDailyFlows(advertiserId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("账户流水========获取完成：accountId：{}", advertiserId);
        }

    }


    /**
     * 快手-账户余额
     *
     * @param advertiserId
     */
    @Override
    public void getKuaishouDailyFundGet(Long advertiserId, String accessToken) {
        log.info("获取账户余额========，账户id：{}", advertiserId);
        String url = postUrl + KuaishouConstant.FUND_GET;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("advertiser_id", advertiserId);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手账户余额======异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONObject details = resultJson.getJSONObject("data");
        if (null == details || details.size() <= 0) {
            log.error("快手账户余额=====返回详情为空，accountId：{}", advertiserId);
            return;
        }
        details.put("advertiser_id", advertiserId);
        KuaishouAdvertiserFundGet reportDaily = KuaishouAdvertiserFundGet.getKuaishouFundGetInfo(details);
        if (!Check.isNull(reportDaily)) {
            fundDailyFlowsMapper.replaceBatchKsFundGetBalance(reportDaily);
        }
        log.info("账户余额========获取完成：accountId：{}", advertiserId);

    }

    @Override
    public List<JSONObject> getAllAccounts() {
        return fundDailyFlowsMapper.getAllAccounts();
    }

    /**
     * 快手-代理商数据
     */
    @Override
    public void getAgentAccountList() {
        List<Long> appList = new ArrayList<>();
        appList.add(27L);
        appList.add(165893351L);
        for (int i = 0; i < appList.size(); i++) {
            Long appId = appList.get(i);
            OauthAgentToken agentToken = agentTokenMapper.getTokenByAppId(appId);
            getAgentAccountList(agentToken.getAccessToken(), agentToken.getAgentId(), 1);
        }

    }


    /**
     * 代理商获取广告主列表
     */
    private void getAgentAccountList(String token, Long agentId, int page) {
        try {
            log.info("=====代理商获取广告主列表====page--{}===《《《《《", page);
            Map<String, String> headers = new HashMap<String, String>();
            headers.put("Content-Type", " application/json");
            headers.put("Access-Token", token);
            JSONObject param = new JSONObject();
            param.put("select_type", 0);
            param.put("select_value", null);
            param.put("agent_id", agentId);
            param.put("page_size", 1000);
            param.put("page", page);

            String result = HttpUtils.httpPostRequest(postUrl + KuaishouConstant.AGENT_ACCOUNT_LIST, param, headers);
            JSONObject resultJson = JSONObject.parseObject(result);
            if (Check.isNull(resultJson)) {
                log.info("代理商获取广告主列表返回为空");
                return;
            }
            Integer code = resultJson.getInteger("code");
            if (code != 0) {
                log.error("代理商获取广告主列表code错误，返回：{}", resultJson);
                return;
            }
            JSONObject data = resultJson.getJSONObject("data");
            if (Check.isNull(data)) {
                log.error("代理商获取广告主列表data节点为空");
                return;
            }
            JSONArray details = data.getJSONArray("details");
            if (Check.isNull(details)) {
                log.error("代理商获取广告主列表details节点为空");
                return;
            }

            List<KuaishouAgentAccountList> agentList = new ArrayList<>();
            for (int i = 0; i < details.size(); i++) {
                JSONObject detailJson = details.getJSONObject(i);
                if (!Check.isNull(detailJson)) {
                    KuaishouAgentAccountList agent = new KuaishouAgentAccountList(detailJson, agentId);
                    agentList.add(agent);
                }
            }
            if (!Check.isNull(agentList)) {
                adUnitReportMapper.replaceBatchKuaishouAgentAccountList(agentList);
            }
            //是否有下一页
            Boolean toGet = true;
            if (details.size() < 1000) {
                toGet = false;
            }
            if (toGet) {
                getAgentAccountList(token, agentId, page + 1);
            } else {
                log.info("代理商获取广告主列表完成========");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
