package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.constant.KuaishouConstant;
import cn.com.ctop.job.kuaishou.data.entity.CompassBindAdvertisers;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAdUnitList;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouCampaignReportDaily;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouMaterialVideoReportDaily;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouUnitReportDaily;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouVideoList;
import cn.com.ctop.job.kuaishou.data.enums.MaterialEnum;
import cn.com.ctop.job.kuaishou.data.mapper.CompassBindAdvertisersMapper;
import cn.com.ctop.job.kuaishou.data.service.CompassBindAdvertisersService;
import cn.com.ctop.job.kuaishou.data.service.IAdUnitReportService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import cn.com.ctop.job.kuaishou.utils.RedisUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class CompassBindAdvertisersServiceImpl extends ServiceImpl<CompassBindAdvertisersMapper, CompassBindAdvertisers> implements CompassBindAdvertisersService {

    @Value("${api.kuaishou.postUrl}")
    private String postUrl;

    @Autowired
    private IAdUnitReportService unitReportService;

    @Override
    public void openapiGwUcV1Advertisers(Long advertiserId, String token) {
        String url = postUrl + KuaishouConstant.GW_UC_V1_ADVERTISERS;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", token);
        Map<String, Object> param = new HashMap<>();
        param.put("advertiser_id", advertiserId);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            String message = resultJson.getString("message");
            log.error("获取[罗盘广告主]列异常:{},advertiserId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");

        if (!Check.isNull(details)) {
            for (int i = 0; i < details.size(); i++) {
                JSONObject jsonObject = details.getJSONObject(i);
                jsonObject.put("user_id", advertiserId);

            }

            baseMapper.replaceBatchAdvertisers(details);
        }
        log.info("[罗盘广告主]执行完成");
    }


    @Override
    public void getCompassAdvertisersCampaignReportDaily(Long advertiserId, String accessToken,
                                                         String startDate, String endDate, int page) {
        String url = postUrl + KuaishouConstant.CAMPAIGN_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "DAILY");
        Integer page_size = 2000;
        param.put("page_size", page_size);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手[罗盘广告主]计划日报表异常:{},accountId:{}", message, advertiserId);
            return;
        }
//        log.info("短剧返回:{}",resultJson);

        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手[罗盘广告主]计划日报返回详情为空，accountId：{}", advertiserId);
            return;
        }
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        List<KuaishouCampaignReportDaily> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouCampaignReportDaily reportDaily = new KuaishouCampaignReportDaily();
            reportDaily.setAdvertiserId(advertiserId);
            reportDaily.setStatDate(DateUtils.getDateInteger(returnJson.getString("stat_date")));
            reportDaily.setCampaignId(returnJson.getLong("campaign_id"));
            reportDaily.setCampaignName(returnJson.getString("campaign_name"));
            reportDaily.setCharge(returnJson.getBigDecimal("charge"));
            reportDaily.setShow(returnJson.getLong("show"));
            reportDaily.setPhotoClick(returnJson.getLong("photo_click"));
            reportDaily.setAclick(returnJson.getLong("aclick"));
            reportDaily.setBclick(returnJson.getLong("bclick"));
            reportDaily.setPhotoClickRatio(returnJson.getBigDecimal("photo_click_ratio"));
            reportDaily.setPlay3sRatio(returnJson.getBigDecimal("play_3s_ratio"));
            reportDaily.setActionRatio(returnJson.getBigDecimal("action_ratio"));
            reportDaily.setImpression1kCost(returnJson.getBigDecimal("impression_1k_cost"));
            reportDaily.setPhotoClickCost(returnJson.getBigDecimal("photo_click_cost"));
            reportDaily.setActionCost(returnJson.getBigDecimal("action_cost"));
            reportDaily.setShare(returnJson.getLong("share"));
            reportDaily.setComment(returnJson.getLong("comment"));
            reportDaily.setLike(returnJson.getLong("like"));
            reportDaily.setFollow(returnJson.getLong("follow"));
            reportDaily.setCancelFollow(returnJson.getLong("cancel_follow"));
            reportDaily.setReport(returnJson.getLong("report"));
            reportDaily.setBlock(returnJson.getLong("block"));
            reportDaily.setNegative(returnJson.getLong("negative"));
            reportDaily.setSubmit(returnJson.getLong("submit"));
            reportDaily.setDownloadStarted(returnJson.getLong("download_started"));
            reportDaily.setDownloadCompleted(returnJson.getLong("download_completed"));
            reportDaily.setActivation(returnJson.getLong("activation"));
            reportDaily.setEventPayFirstDay(returnJson.getLong("event_pay_first_day"));
            reportDaily.setEventPayPurchaseAmountFirstDay(returnJson.getBigDecimal("event_pay_purchase_amount_first_day"));
            reportDaily.setEventPayFirstDayRoi(returnJson.getBigDecimal("event_pay_first_day_roi"));
            reportDaily.setEventPay(returnJson.getLong("event_pay"));
            reportDaily.setEventPayPurchaseAmount(returnJson.getBigDecimal("event_pay_purchase_amount"));
            reportDaily.setEventPayRoi(returnJson.getBigDecimal("event_pay_roi"));
            reportDaily.setEventRegister(returnJson.getLong("event_register"));
            reportDaily.setEventRegisterCost(returnJson.getBigDecimal("event_register_cost"));
            reportDaily.setEventRegisterRatio(returnJson.getBigDecimal("event_register_ratio"));
            reportDaily.setEventJinJianApp(returnJson.getLong("event_jin_jian_app"));
            reportDaily.setEventJinJianAppCost(returnJson.getBigDecimal("event_jin_jian_app_cost"));
            reportDaily.setEventCreditGrantApp(returnJson.getLong("event_credit_grant_app"));
            reportDaily.setEventCreditGrantAppCost(returnJson.getBigDecimal("event_credit_grant_app_cost"));
            reportDaily.setEventCreditGrantAppRatio(returnJson.getBigDecimal("event_credit_grant_app_ratio"));
            reportDaily.setEventOrderPaid(returnJson.getLong("event_order_paid"));
            reportDaily.setEventOrderPaidPurchaseAmount(returnJson.getBigDecimal("event_order_paid_purchase_amount"));
            reportDaily.setEventOrderPaidCost(returnJson.getBigDecimal("event_order_paid_cost"));
            reportDaily.setEventNextDayStay(returnJson.getLong("event_next_day_stay"));
            reportDaily.setEventNextDayStayCost(returnJson.getBigDecimal("event_next_day_stay_cost"));
            reportDaily.setEventNextDayStayRatio(returnJson.getBigDecimal("event_next_day_stay_ratio"));
            reportDaily.setFormCount(returnJson.getLong("form_count"));
            reportDaily.setFormCost(returnJson.getBigDecimal("form_cost"));
            reportDaily.setFormActionRatio(returnJson.getBigDecimal("form_action_ratio"));
            reportDaily.setEventJinJianLandingPage(returnJson.getLong("event_jin_jian_landing_page"));
            reportDaily.setEventJinJianLandingPageCost(returnJson.getBigDecimal("event_jin_jian_landing_page_cost"));
            reportDaily.setEventCreditGrantLandingPage(returnJson.getLong("event_credit_grant_landing_page"));
            reportDaily.setEventCreditGrantLandingPageCost(returnJson.getBigDecimal("event_credit_grant_landing_page_cost"));
            reportDaily.setEventCreditGrantLandingPageRatio(returnJson.getBigDecimal("event_credit_grant_landing_page_ratio"));
            reportDaily.setEventValidClues(returnJson.getLong("event_valid_clues"));
            reportDaily.setEventValidCluesCost(returnJson.getBigDecimal("event_valid_clues_cost"));
            reportDaily.setEventAddWechat(returnJson.getLong("event_add_wechat"));
            reportDaily.setEventAddWechatCost(returnJson.getBigDecimal("event_add_wechat_cost"));
            reportDaily.setEventAddWechatRatio(returnJson.getBigDecimal("event_add_wechat_ratio"));
            reportDaily.setEventGetThrough(returnJson.getLong("event_get_through"));
            reportDaily.setEventGetThroughCost(returnJson.getBigDecimal("event_get_through_cost"));
            reportDaily.setEventGetThroughRatio(returnJson.getBigDecimal("event_get_through_ratio"));
            reportDaily.setEventAppInvoked(returnJson.getLong("event_app_invoked"));
            reportDaily.setEventAppInvokedCost(returnJson.getBigDecimal("event_app_invoked_cost"));
            reportDaily.setEventAppInvokedRatio(returnJson.getBigDecimal("event_app_invoked_ratio"));
            reportDaily.setEventCreditGrantLandingRatio(returnJson.getBigDecimal("event_credit_grant_landing_ratio"));
            reportDaily.setPlay5sRatio(returnJson.getBigDecimal("play_5s_ratio"));
            reportDaily.setPlayEndRatio(returnJson.getBigDecimal("play_end_ratio"));
            reportDaily.setEventNewUserPay(returnJson.getBigDecimal("event_new_user_pay"));
            reportDaily.setEventNewUserPayCost(returnJson.getBigDecimal("event_new_user_pay_cost"));
            reportDaily.setEventNewUserPayRatio(returnJson.getBigDecimal("event_new_user_pay_ratio"));
            reportDaily.setClick1kCost(returnJson.getBigDecimal("click_1k_cost"));
            reportDaily.setAdProductCnt(returnJson.getLong("ad_product_cnt"));
            reportDaily.setEventGoodsView(returnJson.getLong("event_goods_view"));
            reportDaily.setMerchantRecoFans(returnJson.getLong("merchant_reco_fans"));
            reportDaily.setEventGoodsViewCost(returnJson.getBigDecimal("event_goods_view_cost"));
            reportDaily.setMerchantRecoFansCost(returnJson.getBigDecimal("merchant_reco_fans_cost"));
            reportDaily.setEventOrderAmountRoi(returnJson.getBigDecimal("event_order_amount_roi"));
            reportDaily.setEventButtonClick(returnJson.getLong("event_button_click"));
            reportDaily.setEventButtonClickCost(returnJson.getBigDecimal("event_button_click_cost"));
            reportDaily.setEventButtonClickRatio(returnJson.getBigDecimal("event_button_click_ratio"));
            reportDaily.setEventOrderPaidRoi(returnJson.getBigDecimal("event_order_paid_roi"));
            reportDaily.setEventNewUserJinjianApp(returnJson.getLong("event_new_user_jinjian_app"));
            reportDaily.setEventNewUserJinjianAppCost(returnJson.getBigDecimal("event_new_user_jinjian_app_cost"));
            reportDaily.setEventNewUserJinjianAppRoi(returnJson.getBigDecimal("event_new_user_jinjian_app_roi"));
            reportDaily.setEventNewUserCreditGrantApp(returnJson.getLong("event_new_user_credit_grant_app"));
            reportDaily.setEventNewUserCreditGrantAppCost(returnJson.getBigDecimal("event_new_user_credit_grant_app_cost"));
            reportDaily.setEventNewUserCreditGrantAppRoi(returnJson.getBigDecimal("event_new_user_credit_grant_app_roi"));
            reportDaily.setEventNewUserJinjianPage(returnJson.getLong("event_new_user_jinjian_page"));
            reportDaily.setEventNewUserJinjianPageCost(returnJson.getBigDecimal("event_new_user_jinjian_page_cost"));
            reportDaily.setEventNewUserJinjianPageRoi(returnJson.getBigDecimal("event_new_user_jinjian_page_roi"));
            reportDaily.setEventNewUserCreditGrantPage(returnJson.getLong("event_new_user_credit_grant_page"));
            reportDaily.setEventNewUserCreditGrantPageCost(returnJson.getBigDecimal("event_new_user_credit_grant_page_cost"));
            reportDaily.setEventNewUserCreditGrantPageRoi(returnJson.getBigDecimal("event_new_user_credit_grant_page_roi"));
            reportDaily.setEventAppointForm(returnJson.getLong("event_appoint_form"));
            reportDaily.setEventAppointFormCost(returnJson.getBigDecimal("event_appoint_form_cost"));
            reportDaily.setEventAppointFormRatio(returnJson.getBigDecimal("event_appoint_form_ratio"));
            reportDaily.setEventAppointJumpClick(returnJson.getLong("event_appoint_jump_click"));
            reportDaily.setEventAppointJumpClickCost(returnJson.getBigDecimal("event_appoint_jump_click_cost"));
            reportDaily.setEventAppointJumpClickRatio(returnJson.getBigDecimal("event_appoint_jump_click_ratio"));
            reportDaily.setUnionEventPayPurchaseAmount7d(returnJson.getLong("union_event_pay_purchase_amount_7d"));
            reportDaily.setUnionEventPayPurchaseAmount7dRoi(returnJson.getBigDecimal("union_event_pay_purchase_amount_7d_roi"));
            reportDaily.setPlacementType(returnJson.getString("placement_type"));
            reportDaily.setAdScene(returnJson.getString("ad_scene"));
            reportDaily.setEventOrderSuccessed(returnJson.getLong("event_order_successed"));
            reportDaily.setAdPhotoPlayed10sRatio(returnJson.getBigDecimal("ad_photo_played_10s_ratio"));
            reportDaily.setKeyActionCost(returnJson.getBigDecimal("key_action_cost"));
            reportDaily.setEventAddShoppingCartCost(returnJson.getBigDecimal("event_add_shopping_cart_cost"));
            reportDaily.setEventAdWatchTimesRatio(returnJson.getBigDecimal("event_ad_watch_times_ratio"));
            reportDaily.setEventOutboundCallCost(returnJson.getBigDecimal("event_outbound_call_cost"));
            reportDaily.setEventOutboundCall(returnJson.getLong("event_outbound_call"));
            reportDaily.setActionNewRatio(returnJson.getBigDecimal("action_new_ratio"));
            reportDaily.setEventWatchAppAd(returnJson.getLong("event_watch_app_ad"));
            reportDaily.setEventMultiConversionCost(returnJson.getBigDecimal("event_multi_conversion_cost"));
            reportDaily.setEventPhoneCardActivate(returnJson.getLong("event_phone_card_activate"));
            reportDaily.setAdPhotoPlayed75percentRatio(returnJson.getBigDecimal("ad_photo_played_75percent_ratio"));
            //   reportDaily.setKeyActionRatio(returnJson.getBigDecimal("key_action_ratio"));
            reportDaily.setEventAdWatchTimesCost(returnJson.getBigDecimal("event_ad_watch_times_cost"));
            reportDaily.setEventAddShoppingCart(returnJson.getLong("event_add_shopping_cart"));
            reportDaily.setKeyAction(returnJson.getLong("key_action"));
            reportDaily.setEventMultiConversion(returnJson.getLong("event_multi_conversion"));
            reportDaily.setEventWechatConnected(returnJson.getLong("event_wechat_connected"));
            reportDaily.setEventDspGiftForm(returnJson.getLong("event_dsp_gift_form"));
            reportDaily.setEventIntentionConfirmed(returnJson.getLong("event_intention_confirmed"));
            reportDaily.setEventPhoneGetThrough(returnJson.getLong("event_phone_get_through"));
            reportDaily.setEventMultiConversionRatio(returnJson.getBigDecimal("event_multi_conversion_ratio"));
            reportDaily.setEventMeasurementHouse(returnJson.getBigDecimal("event_measurement_house"));
            reportDaily.setAdPhotoPlayed2sRatio(returnJson.getBigDecimal("ad_photo_played_2s_ratio"));
            reportDaily.setEventOutboundCallRatio(returnJson.getBigDecimal("event_outbound_call_ratio"));
            reportDaily.setEventAdWatchTimes(returnJson.getLong("event_ad_watch_times"));
            reportDaily.setMinigameIaaPurchaseAmount(returnJson.getBigDecimal("minigame_iaa_purchase_amount"));
            reportDaily.setMinigameIaaPurchaseRoi(returnJson.getBigDecimal("minigame_iaa_purchase_roi"));
            reportDaily.setEventPayPurchaseAmountOneDay(returnJson.getBigDecimal("event_pay_purchase_amount_one_day"));
            addList.add(reportDaily);
        }
        if (!Check.isNull(addList)) {
            baseMapper.replaceBatchCampaignReportDaily(addList);
        }
        if (toGet) {
            getCompassAdvertisersCampaignReportDaily(advertiserId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("[罗盘广告主]计划日报数据获取完成：accountId：{}", advertiserId);
        }
    }


    @Override
    public void getCompassAdvertisersUnitReportDaily(Long advertiserId, String accessToken, String startDate, String endDate, int page) {
        String url = postUrl + KuaishouConstant.UNIT_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "DAILY");
        Integer page_size = 2000;
        param.put("page_size", page_size);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手[罗盘广告主]广告组日报表异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("[罗盘广告主]快手组日报返回详情为空，accountId：{}", advertiserId);
            return;
        }
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        List<KuaishouUnitReportDaily> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouUnitReportDaily reportDaily = new KuaishouUnitReportDaily();
            reportDaily.setAdvertiserId(advertiserId);
            reportDaily.setStatDate(DateUtils.getDateInteger(returnJson.getString("stat_date")));
            reportDaily.setCampaignId(returnJson.getLong("campaign_id"));
            reportDaily.setCampaignName(returnJson.getString("campaign_name"));
            reportDaily.setUnitId(returnJson.getLong("unit_id"));
            reportDaily.setUnitName(returnJson.getString("unit_name"));
            reportDaily.setCharge(returnJson.getBigDecimal("charge"));
            reportDaily.setShow(returnJson.getLong("show"));
            reportDaily.setPhotoClick(returnJson.getLong("photo_click"));
            reportDaily.setAclick(returnJson.getLong("aclick"));
            reportDaily.setBclick(returnJson.getLong("bclick"));
            reportDaily.setPhotoClickRatio(returnJson.getBigDecimal("photo_click_ratio"));
            reportDaily.setPlay3sRatio(returnJson.getBigDecimal("play_3s_ratio"));
            reportDaily.setActionRatio(returnJson.getBigDecimal("action_ratio"));
            reportDaily.setImpression1kCost(returnJson.getBigDecimal("impression_1k_cost"));
            reportDaily.setPhotoClickCost(returnJson.getBigDecimal("photo_click_cost"));
            reportDaily.setActionCost(returnJson.getBigDecimal("action_cost"));
            reportDaily.setShare(returnJson.getLong("share"));
            reportDaily.setComment(returnJson.getLong("comment"));
            reportDaily.setLike(returnJson.getLong("like"));
            reportDaily.setFollow(returnJson.getLong("follow"));
            reportDaily.setCancelFollow(returnJson.getLong("cancel_follow"));
            reportDaily.setReport(returnJson.getLong("report"));
            reportDaily.setBlock(returnJson.getLong("block"));
            reportDaily.setNegative(returnJson.getLong("negative"));
            reportDaily.setSubmit(returnJson.getLong("submit"));
            reportDaily.setDownloadStarted(returnJson.getLong("download_started"));
            reportDaily.setDownloadCompleted(returnJson.getLong("download_completed"));
            reportDaily.setActivation(returnJson.getLong("activation"));
            reportDaily.setEventPayFirstDay(returnJson.getLong("event_pay_first_day"));
            reportDaily.setEventPayPurchaseAmountFirstDay(returnJson.getBigDecimal("event_pay_purchase_amount_first_day"));
            reportDaily.setEventPayFirstDayRoi(returnJson.getBigDecimal("event_pay_first_day_roi"));
            reportDaily.setEventPay(returnJson.getLong("event_pay"));
            reportDaily.setEventPayPurchaseAmount(returnJson.getBigDecimal("event_pay_purchase_amount"));
            reportDaily.setEventPayRoi(returnJson.getBigDecimal("event_pay_roi"));
            reportDaily.setEventRegister(returnJson.getLong("event_register"));
            reportDaily.setEventRegisterCost(returnJson.getBigDecimal("event_register_cost"));
            reportDaily.setEventRegisterRatio(returnJson.getBigDecimal("event_register_ratio"));
            reportDaily.setEventJinJianApp(returnJson.getLong("event_jin_jian_app"));
            reportDaily.setEventJinJianAppCost(returnJson.getBigDecimal("event_jin_jian_app_cost"));
            reportDaily.setEventCreditGrantApp(returnJson.getLong("event_credit_grant_app"));
            reportDaily.setEventCreditGrantAppCost(returnJson.getBigDecimal("event_credit_grant_app_cost"));
            reportDaily.setEventCreditGrantAppRatio(returnJson.getBigDecimal("event_credit_grant_app_ratio"));
            reportDaily.setEventOrderPaid(returnJson.getLong("event_order_paid"));
            reportDaily.setEventOrderPaidPurchaseAmount(returnJson.getBigDecimal("event_order_paid_purchase_amount"));
            reportDaily.setEventOrderPaidCost(returnJson.getBigDecimal("event_order_paid_cost"));
            reportDaily.setEventNextDayStay(returnJson.getLong("event_next_day_stay"));
            reportDaily.setEventNextDayStayCost(returnJson.getBigDecimal("event_next_day_stay_cost"));
            reportDaily.setEventNextDayStayRatio(returnJson.getBigDecimal("event_next_day_stay_ratio"));
            reportDaily.setFormCount(returnJson.getLong("form_count"));
            reportDaily.setFormCost(returnJson.getBigDecimal("form_cost"));
            reportDaily.setFormActionRatio(returnJson.getBigDecimal("form_action_ratio"));
            reportDaily.setEventJinJianLandingPage(returnJson.getLong("event_jin_jian_landing_page"));
            reportDaily.setEventJinJianLandingPageCost(returnJson.getBigDecimal("event_jin_jian_landing_page_cost"));
            reportDaily.setEventCreditGrantLandingPage(returnJson.getLong("event_credit_grant_landing_page"));
            reportDaily.setEventCreditGrantLandingPageCost(returnJson.getBigDecimal("event_credit_grant_landing_page_cost"));
            reportDaily.setEventCreditGrantLandingPageRatio(returnJson.getBigDecimal("event_credit_grant_landing_page_ratio"));
            reportDaily.setEventValidClues(returnJson.getLong("event_valid_clues"));
            reportDaily.setEventValidCluesCost(returnJson.getBigDecimal("event_valid_clues_cost"));
            reportDaily.setEventAddWechat(returnJson.getLong("event_add_wechat"));
            reportDaily.setEventAddWechatCost(returnJson.getBigDecimal("event_add_wechat_cost"));
            reportDaily.setEventAddWechatRatio(returnJson.getBigDecimal("event_add_wechat_ratio"));
            reportDaily.setEventGetThrough(returnJson.getLong("event_get_through"));
            reportDaily.setEventGetThroughCost(returnJson.getBigDecimal("event_get_through_cost"));
            reportDaily.setEventGetThroughRatio(returnJson.getBigDecimal("event_get_through_ratio"));
            reportDaily.setEventAppInvoked(returnJson.getLong("event_app_invoked"));
            reportDaily.setEventAppInvokedCost(returnJson.getBigDecimal("event_app_invoked_cost"));
            reportDaily.setEventAppInvokedRatio(returnJson.getBigDecimal("event_app_invoked_ratio"));
            reportDaily.setEventCreditGrantLandingRatio(returnJson.getBigDecimal("event_credit_grant_landing_ratio"));
            reportDaily.setPlay5sRatio(returnJson.getBigDecimal("play_5s_ratio"));
            reportDaily.setPlayEndRatio(returnJson.getBigDecimal("play_end_ratio"));
            reportDaily.setEventNewUserPay(returnJson.getBigDecimal("event_new_user_pay"));
            reportDaily.setEventNewUserPayCost(returnJson.getBigDecimal("event_new_user_pay_cost"));
            reportDaily.setEventNewUserPayRatio(returnJson.getBigDecimal("event_new_user_pay_ratio"));
            reportDaily.setClick1kCost(returnJson.getBigDecimal("click_1k_cost"));
            reportDaily.setAdProductCnt(returnJson.getLong("ad_product_cnt"));
            reportDaily.setEventGoodsView(returnJson.getLong("event_goods_view"));
            reportDaily.setMerchantRecoFans(returnJson.getLong("merchant_reco_fans"));
            reportDaily.setEventGoodsViewCost(returnJson.getBigDecimal("event_goods_view_cost"));
            reportDaily.setMerchantRecoFansCost(returnJson.getBigDecimal("merchant_reco_fans_cost"));
            reportDaily.setEventOrderAmountRoi(returnJson.getBigDecimal("event_order_amount_roi"));
            reportDaily.setEventButtonClick(returnJson.getLong("event_button_click"));
            reportDaily.setEventButtonClickCost(returnJson.getBigDecimal("event_button_click_cost"));
            reportDaily.setEventButtonClickRatio(returnJson.getBigDecimal("event_button_click_ratio"));
            reportDaily.setEventOrderPaidRoi(returnJson.getBigDecimal("event_order_paid_roi"));
            reportDaily.setEventNewUserJinjianApp(returnJson.getLong("event_new_user_jinjian_app"));
            reportDaily.setEventNewUserJinjianAppCost(returnJson.getBigDecimal("event_new_user_jinjian_app_cost"));
            reportDaily.setEventNewUserJinjianAppRoi(returnJson.getBigDecimal("event_new_user_jinjian_app_roi"));
            reportDaily.setEventNewUserCreditGrantApp(returnJson.getLong("event_new_user_credit_grant_app"));
            reportDaily.setEventNewUserCreditGrantAppCost(returnJson.getBigDecimal("event_new_user_credit_grant_app_cost"));
            reportDaily.setEventNewUserCreditGrantAppRoi(returnJson.getBigDecimal("event_new_user_credit_grant_app_roi"));
            reportDaily.setEventNewUserJinjianPage(returnJson.getLong("event_new_user_jinjian_page"));
            reportDaily.setEventNewUserJinjianPageCost(returnJson.getBigDecimal("event_new_user_jinjian_page_cost"));
            reportDaily.setEventNewUserJinjianPageRoi(returnJson.getBigDecimal("event_new_user_jinjian_page_roi"));
            reportDaily.setEventNewUserCreditGrantPage(returnJson.getLong("event_new_user_credit_grant_page"));
            reportDaily.setEventNewUserCreditGrantPageCost(returnJson.getBigDecimal("event_new_user_credit_grant_page_cost"));
            reportDaily.setEventNewUserCreditGrantPageRoi(returnJson.getBigDecimal("event_new_user_credit_grant_page_roi"));
            reportDaily.setEventAppointForm(returnJson.getLong("event_appoint_form"));
            reportDaily.setEventAppointFormCost(returnJson.getBigDecimal("event_appoint_form_cost"));
            reportDaily.setEventAppointFormRatio(returnJson.getBigDecimal("event_appoint_form_ratio"));
            reportDaily.setEventAppointJumpClick(returnJson.getLong("event_appoint_jump_click"));
            reportDaily.setEventAppointJumpClickCost(returnJson.getBigDecimal("event_appoint_jump_click_cost"));
            reportDaily.setEventAppointJumpClickRatio(returnJson.getBigDecimal("event_appoint_jump_click_ratio"));
            reportDaily.setUnionEventPayPurchaseAmount7d(returnJson.getLong("union_event_pay_purchase_amount_7d"));
            reportDaily.setUnionEventPayPurchaseAmount7dRoi(returnJson.getBigDecimal("union_event_pay_purchase_amount_7d_roi"));
            reportDaily.setPlacementType(returnJson.getString("placement_type"));
            reportDaily.setAdScene(returnJson.getString("ad_scene"));
            reportDaily.setEventOrderSuccessed(returnJson.getLong("event_order_successed"));
            reportDaily.setAdPhotoPlayed10sRatio(returnJson.getBigDecimal("ad_photo_played_10s_ratio"));
            reportDaily.setKeyActionCost(returnJson.getBigDecimal("key_action_cost"));
            reportDaily.setEventAddShoppingCartCost(returnJson.getBigDecimal("event_add_shopping_cart_cost"));
            reportDaily.setEventAdWatchTimesRatio(returnJson.getBigDecimal("event_ad_watch_times_ratio"));
            reportDaily.setEventOutboundCallCost(returnJson.getBigDecimal("event_outbound_call_cost"));
            reportDaily.setEventOutboundCall(returnJson.getLong("event_outbound_call"));
            reportDaily.setActionNewRatio(returnJson.getBigDecimal("action_new_ratio"));
            reportDaily.setEventWatchAppAd(returnJson.getLong("event_watch_app_ad"));
            reportDaily.setEventMultiConversionCost(returnJson.getBigDecimal("event_multi_conversion_cost"));
            reportDaily.setEventPhoneCardActivate(returnJson.getLong("event_phone_card_activate"));
            reportDaily.setAdPhotoPlayed75percentRatio(returnJson.getBigDecimal("ad_photo_played_75percent_ratio"));
            //   reportDaily.setKeyActionRatio(returnJson.getBigDecimal("key_action_ratio"));
            reportDaily.setEventAdWatchTimesCost(returnJson.getBigDecimal("event_ad_watch_times_cost"));
            reportDaily.setEventAddShoppingCart(returnJson.getLong("event_add_shopping_cart"));
            reportDaily.setKeyAction(returnJson.getLong("key_action"));
            reportDaily.setEventMultiConversion(returnJson.getLong("event_multi_conversion"));
            reportDaily.setEventWechatConnected(returnJson.getLong("event_wechat_connected"));
            reportDaily.setEventDspGiftForm(returnJson.getLong("event_dsp_gift_form"));
            reportDaily.setEventIntentionConfirmed(returnJson.getLong("event_intention_confirmed"));
            reportDaily.setEventPhoneGetThrough(returnJson.getLong("event_phone_get_through"));
            reportDaily.setEventMultiConversionRatio(returnJson.getBigDecimal("event_multi_conversion_ratio"));
            reportDaily.setEventMeasurementHouse(returnJson.getBigDecimal("event_measurement_house"));
            reportDaily.setAdPhotoPlayed2sRatio(returnJson.getBigDecimal("ad_photo_played_2s_ratio"));
            reportDaily.setEventOutboundCallRatio(returnJson.getBigDecimal("event_outbound_call_ratio"));
            reportDaily.setEventAdWatchTimes(returnJson.getLong("event_ad_watch_times"));
            reportDaily.setEventOrderSubmit(returnJson.getLong("event_order_submit"));
            reportDaily.setLiveEventGoodsView(returnJson.getLong("live_event_goods_view"));
            reportDaily.setMinigameIaaPurchaseAmount(returnJson.getBigDecimal("minigame_iaa_purchase_amount"));
            reportDaily.setMinigameIaaPurchaseRoi(returnJson.getBigDecimal("minigame_iaa_purchase_roi"));
            addList.add(reportDaily);
        }
        if (!Check.isNull(addList)) {
            baseMapper.replaceBatchUnitReportDaily(addList);
        }
        if (toGet) {
            getCompassAdvertisersUnitReportDaily(advertiserId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("罗盘广告主-组日报数据获取完成：accountId：{}", advertiserId);
        }
    }

    @Override
    public void getCompassAdvertisersVideosReportDaily(Long advertiserId, String accessToken, String startDate, String endDate, int page) {
        String url = postUrl + KuaishouConstant.MATERIAL_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "DAILY");
        param.put("view_type", 5);
        Integer page_size = 2000;
        param.put("page_size", page_size);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        JSONArray details = resultJson.getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手[罗盘广告主]视频日报返回详情为空，accountId：{}", advertiserId);
            return;
        }
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        List<KuaishouMaterialVideoReportDaily> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouMaterialVideoReportDaily reportDaily = new KuaishouMaterialVideoReportDaily();
            reportDaily.setAdvertiserId(advertiserId);
            reportDaily.setStatDate(DateUtils.getDateInteger(returnJson.getString("stat_date")));
            reportDaily.setPhotoId(returnJson.getString("photo_id"));
            reportDaily.setAdScene(returnJson.getString("ad_scene"));
            reportDaily.setPhotoUrl(returnJson.getString("photo_url"));
            reportDaily.setCoverUrl(returnJson.getString("cover_url"));
            reportDaily.setImageToken(returnJson.getString("image_token"));
            reportDaily.setPhotoCaption("");
            reportDaily.setCharge(returnJson.getBigDecimal("charge"));
            reportDaily.setShow(returnJson.getLong("show"));
            reportDaily.setPhotoClick(returnJson.getLong("photo_click"));
            reportDaily.setAclick(returnJson.getLong("aclick"));
            reportDaily.setBclick(returnJson.getLong("bclick"));
            reportDaily.setPhotoClickRatio(returnJson.getBigDecimal("photo_click_ratio"));
            reportDaily.setPlay3sRatio(returnJson.getBigDecimal("play_3s_ratio"));
            reportDaily.setActionRatio(returnJson.getBigDecimal("action_ratio"));
            reportDaily.setImpression1kCost(returnJson.getBigDecimal("impression_1k_cost"));
            reportDaily.setPhotoClickCost(returnJson.getBigDecimal("photo_click_cost"));
            reportDaily.setActionCost(returnJson.getBigDecimal("action_cost"));
            reportDaily.setShare(returnJson.getLong("share"));
            reportDaily.setComment(returnJson.getLong("comment"));
            reportDaily.setLike(returnJson.getLong("like"));
            reportDaily.setFollow(returnJson.getLong("follow"));
            reportDaily.setCancelFollow(returnJson.getLong("cancel_follow"));
            reportDaily.setReport(returnJson.getLong("report"));
            reportDaily.setBlock(returnJson.getLong("block"));
            reportDaily.setNegative(returnJson.getLong("negative"));
            reportDaily.setSubmit(returnJson.getLong("submit"));
            reportDaily.setDownloadStarted(returnJson.getLong("download_started"));
            reportDaily.setDownloadCompleted(returnJson.getLong("download_completed"));
            reportDaily.setActivation(returnJson.getLong("activation"));
            reportDaily.setEventPayFirstDay(returnJson.getLong("event_pay_first_day"));
            reportDaily.setEventPayPurchaseAmountFirstDay(returnJson.getBigDecimal("event_pay_purchase_amount_first_day"));
            reportDaily.setEventPayFirstDayRoi(returnJson.getBigDecimal("event_pay_first_day_roi"));
            reportDaily.setEventPay(returnJson.getLong("event_pay"));
            reportDaily.setEventPayPurchaseAmount(returnJson.getBigDecimal("event_pay_purchase_amount"));
            reportDaily.setEventPayRoi(returnJson.getBigDecimal("event_pay_roi"));
            reportDaily.setEventRegister(returnJson.getLong("event_register"));
            reportDaily.setEventRegisterCost(returnJson.getBigDecimal("event_register_cost"));
            reportDaily.setEventRegisterRatio(returnJson.getBigDecimal("event_register_ratio"));
            reportDaily.setEventJinJianApp(returnJson.getLong("event_jin_jian_app"));
            reportDaily.setEventJinJianAppCost(returnJson.getBigDecimal("event_jin_jian_app_cost"));
            reportDaily.setEventCreditGrantApp(returnJson.getLong("event_credit_grant_app"));
            reportDaily.setEventCreditGrantAppCost(returnJson.getBigDecimal("event_credit_grant_app_cost"));
            reportDaily.setEventCreditGrantAppRatio(returnJson.getBigDecimal("event_credit_grant_app_ratio"));
            reportDaily.setEventOrderPaid(returnJson.getLong("event_order_paid"));
            reportDaily.setEventOrderPaidPurchaseAmount(returnJson.getBigDecimal("event_order_paid_purchase_amount"));
            reportDaily.setEventOrderPaidCost(returnJson.getBigDecimal("event_order_paid_cost"));
            reportDaily.setEventNextDayStay(returnJson.getLong("event_next_day_stay"));
            reportDaily.setEventNextDayStayCost(returnJson.getBigDecimal("event_next_day_stay_cost"));
            reportDaily.setEventNextDayStayRatio(returnJson.getBigDecimal("event_next_day_stay_ratio"));
            reportDaily.setFormCount(returnJson.getLong("form_count"));
            reportDaily.setFormCost(returnJson.getBigDecimal("form_cost"));
            reportDaily.setFormActionRatio(returnJson.getBigDecimal("form_action_ratio"));
            reportDaily.setEventJinJianLandingPage(returnJson.getLong("event_jin_jian_landing_page"));
            reportDaily.setEventJinJianLandingPageCost(returnJson.getBigDecimal("event_jin_jian_landing_page_cost"));
            reportDaily.setEventCreditGrantLandingPage(returnJson.getLong("event_credit_grant_landing_page"));
            reportDaily.setEventCreditGrantLandingPageCost(returnJson.getBigDecimal("event_credit_grant_landing_page_cost"));
            reportDaily.setEventCreditGrantLandingPageRatio(returnJson.getBigDecimal("event_credit_grant_landing_page_ratio"));
            reportDaily.setEventValidClues(returnJson.getLong("event_valid_clues"));
            reportDaily.setEventValidCluesCost(returnJson.getBigDecimal("event_valid_clues_cost"));
            reportDaily.setEventAddWechat(returnJson.getLong("event_add_wechat"));
            reportDaily.setEventAddWechatCost(returnJson.getBigDecimal("event_add_wechat_cost"));
            reportDaily.setEventAddWechatRatio(returnJson.getBigDecimal("event_add_wechat_ratio"));
            reportDaily.setEventGetThrough(returnJson.getLong("event_get_through"));
            reportDaily.setEventGetThroughCost(returnJson.getBigDecimal("event_get_through_cost"));
            reportDaily.setEventGetThroughRatio(returnJson.getBigDecimal("event_get_through_ratio"));
            reportDaily.setEventAppInvoked(returnJson.getLong("event_app_invoked"));
            reportDaily.setEventAppInvokedCost(returnJson.getBigDecimal("event_app_invoked_cost"));
            reportDaily.setEventAppInvokedRatio(returnJson.getBigDecimal("event_app_invoked_ratio"));
            reportDaily.setEventCreditGrantLandingRatio(returnJson.getBigDecimal("event_credit_grant_landing_ratio"));
            reportDaily.setPlay5sRatio(returnJson.getBigDecimal("play_5s_ratio"));
            reportDaily.setPlayEndRatio(returnJson.getBigDecimal("play_end_ratio"));
            reportDaily.setEventNewUserPay(returnJson.getBigDecimal("event_new_user_pay"));
            reportDaily.setEventNewUserPayCost(returnJson.getBigDecimal("event_new_user_pay_cost"));
            reportDaily.setEventNewUserPayRatio(returnJson.getBigDecimal("event_new_user_pay_ratio"));
            reportDaily.setClick1kCost(returnJson.getBigDecimal("click_1k_cost"));
            reportDaily.setAdProductCnt(returnJson.getLong("ad_product_cnt"));
            reportDaily.setEventGoodsView(returnJson.getLong("event_goods_view"));
            reportDaily.setMerchantRecoFans(returnJson.getLong("merchant_reco_fans"));
            reportDaily.setEventGoodsViewCost(returnJson.getBigDecimal("event_goods_view_cost"));
            reportDaily.setMerchantRecoFansCost(returnJson.getBigDecimal("merchant_reco_fans_cost"));
            reportDaily.setEventOrderAmountRoi(returnJson.getBigDecimal("event_order_amount_roi"));
            reportDaily.setEventButtonClick(returnJson.getLong("event_button_click"));
            reportDaily.setEventButtonClickCost(returnJson.getBigDecimal("event_button_click_cost"));
            reportDaily.setEventButtonClickRatio(returnJson.getBigDecimal("event_button_click_ratio"));
            reportDaily.setEventOrderPaidRoi(returnJson.getBigDecimal("event_order_paid_roi"));
            reportDaily.setEventNewUserJinjianApp(returnJson.getLong("event_new_user_jinjian_app"));
            reportDaily.setEventNewUserJinjianAppCost(returnJson.getBigDecimal("event_new_user_jinjian_app_cost"));
            reportDaily.setEventNewUserJinjianAppRoi(returnJson.getBigDecimal("event_new_user_jinjian_app_roi"));
            reportDaily.setEventNewUserCreditGrantApp(returnJson.getLong("event_new_user_credit_grant_app"));
            reportDaily.setEventNewUserCreditGrantAppCost(returnJson.getBigDecimal("event_new_user_credit_grant_app_cost"));
            reportDaily.setEventNewUserCreditGrantAppRoi(returnJson.getBigDecimal("event_new_user_credit_grant_app_roi"));
            reportDaily.setEventNewUserJinjianPage(returnJson.getLong("event_new_user_jinjian_page"));
            reportDaily.setEventNewUserJinjianPageCost(returnJson.getBigDecimal("event_new_user_jinjian_page_cost"));
            reportDaily.setEventNewUserJinjianPageRoi(returnJson.getBigDecimal("event_new_user_jinjian_page_roi"));
            reportDaily.setEventNewUserCreditGrantPage(returnJson.getLong("event_new_user_credit_grant_page"));
            reportDaily.setEventNewUserCreditGrantPageCost(returnJson.getBigDecimal("event_new_user_credit_grant_page_cost"));
            reportDaily.setEventNewUserCreditGrantPageRoi(returnJson.getBigDecimal("event_new_user_credit_grant_page_roi"));
            reportDaily.setEventAppointForm(returnJson.getLong("event_appoint_form"));
            reportDaily.setEventAppointFormCost(returnJson.getBigDecimal("event_appoint_form_cost"));
            reportDaily.setEventAppointFormRatio(returnJson.getBigDecimal("event_appoint_form_ratio"));
            reportDaily.setEventAppointJumpClick(returnJson.getLong("event_appoint_jump_click"));
            reportDaily.setEventAppointJumpClickCost(returnJson.getBigDecimal("event_appoint_jump_click_cost"));
            reportDaily.setEventAppointJumpClickRatio(returnJson.getBigDecimal("event_appoint_jump_click_ratio"));
            reportDaily.setUnionEventPayPurchaseAmount7d(returnJson.getLong("union_event_pay_purchase_amount_7d"));
            reportDaily.setUnionEventPayPurchaseAmount7dRoi(returnJson.getBigDecimal("union_event_pay_purchase_amount_7d_roi"));
            reportDaily.setPlacementType(returnJson.getString("placement_type"));
            reportDaily.setEventOrderSuccessed(returnJson.getLong("event_order_successed"));
            reportDaily.setAdPhotoPlayed10sRatio(returnJson.getBigDecimal("ad_photo_played_10s_ratio"));
            reportDaily.setKeyActionCost(returnJson.getBigDecimal("key_action_cost"));
            reportDaily.setEventAddShoppingCartCost(returnJson.getBigDecimal("event_add_shopping_cart_cost"));
            reportDaily.setEventAdWatchTimesRatio(returnJson.getBigDecimal("event_ad_watch_times_ratio"));
            reportDaily.setEventOutboundCallCost(returnJson.getBigDecimal("event_outbound_call_cost"));
            reportDaily.setEventOutboundCall(returnJson.getLong("event_outbound_call"));
            reportDaily.setActionNewRatio(returnJson.getBigDecimal("action_new_ratio"));
            reportDaily.setEventWatchAppAd(returnJson.getLong("event_watch_app_ad"));
            reportDaily.setEventMultiConversionCost(returnJson.getBigDecimal("event_multi_conversion_cost"));
            reportDaily.setEventPhoneCardActivate(returnJson.getLong("event_phone_card_activate"));
            reportDaily.setAdPhotoPlayed75percentRatio(returnJson.getBigDecimal("ad_photo_played_75percent_ratio"));
            //  reportDaily.setKeyActionRatio(returnJson.getBigDecimal("key_action_ratio"));
            reportDaily.setEventAdWatchTimesCost(returnJson.getBigDecimal("event_ad_watch_times_cost"));
            reportDaily.setEventAddShoppingCart(returnJson.getLong("event_add_shopping_cart"));
            reportDaily.setKeyAction(returnJson.getLong("key_action"));
            reportDaily.setEventMultiConversion(returnJson.getLong("event_multi_conversion"));
            reportDaily.setEventWechatConnected(returnJson.getLong("event_wechat_connected"));
            reportDaily.setEventDspGiftForm(returnJson.getLong("event_dsp_gift_form"));
            reportDaily.setEventIntentionConfirmed(returnJson.getLong("event_intention_confirmed"));
            reportDaily.setEventPhoneGetThrough(returnJson.getLong("event_phone_get_through"));
            reportDaily.setEventMultiConversionRatio(returnJson.getBigDecimal("event_multi_conversion_ratio"));
            reportDaily.setEventMeasurementHouse(returnJson.getBigDecimal("event_measurement_house"));
            reportDaily.setAdPhotoPlayed2sRatio(returnJson.getBigDecimal("ad_photo_played_2s_ratio"));
            reportDaily.setEventOutboundCallRatio(returnJson.getBigDecimal("event_outbound_call_ratio"));
            reportDaily.setEventAdWatchTimes(returnJson.getLong("event_ad_watch_times"));
            reportDaily.setMinigameIaaPurchaseAmount(returnJson.getBigDecimal("minigame_iaa_purchase_amount"));
            reportDaily.setMinigameIaaPurchaseRoi(returnJson.getBigDecimal("minigame_iaa_purchase_roi"));

            addList.add(reportDaily);
        }
        baseMapper.replaceBatchVideosportDaily(addList);
        if (toGet) {
            getCompassAdvertisersVideosReportDaily(advertiserId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("[罗盘广告主]视频日报数据获取完成：accountId：{}", advertiserId);
        }
    }

    @Override
    public void checkCompassAdvertisersUnit(Long advertiserId, String accessToken, String startDate, String endDate, int page) {
        String url = postUrl + KuaishouConstant.UNIT_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "DAILY");
        Integer page_size = 2000;
        param.put("page_size", page_size);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("查询快手[罗盘广告主]广告组异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
//            log.error("[罗盘广告主]快手组详情为空，accountId：{}", advertiserId);
            return;
        }
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            BigDecimal charge = returnJson.getBigDecimal("charge").setScale(2, RoundingMode.HALF_UP);
            Long unitId = returnJson.getLong("unit_id");
            BigDecimal payPurchaseAmount = returnJson.getBigDecimal("event_pay_purchase_amount").setScale(2, RoundingMode.HALF_UP);
            if (charge.signum() != 0) {
                //付费金额/消耗
                BigDecimal roi = payPurchaseAmount.divide(charge, 2, RoundingMode.HALF_UP);
                //1、消耗大于30，并且roi（付费金额/消耗）低于0.8的【自动关停】
                if (charge.compareTo(new BigDecimal("30")) > 0 && roi.compareTo(new BigDecimal("0.8")) < 0) {
                    unitReportService.updateUnitStatus(accessToken, advertiserId, unitId, 2);
                    String text = "消耗：" + charge + "，大于30；并且roi(付费金额：" + payPurchaseAmount + " /消耗)：" + roi + "，低于0.8，自动关停！！！";
                    insertRecord(advertiserId, unitId, "关停", text);
                }
                //2、roi 大于1的广告组【自动开启】
                if (roi.compareTo(new BigDecimal("1.0")) > 0) {
                    Integer unitStatus = getAdUnitStatus(advertiserId, unitId, accessToken);
                    if (unitStatus == 2) {
                        //1：投放中；2：暂停 3：删除
                        unitReportService.updateUnitStatus(accessToken, advertiserId, unitId, 1);
                        insertRecord(advertiserId, unitId, "开启", "roi:" + roi + "大于1");

                    }
                }
            }

        }
        if (toGet) {
            getCompassAdvertisersUnitReportDaily(advertiserId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("罗盘广告主-广告组校验完成：accountId：{}", advertiserId);
        }
    }

    private void insertRecord(Long advertiserId, Long unitId, String type, String text) {
        StringBuilder alarmDetail = new StringBuilder();
        alarmDetail.append("账户ID：").append(advertiserId)
                .append("下的广告组ID:").append(unitId).append("，")
                .append(type).append("原因：").append(text);
        baseMapper.insertRecord(advertiserId, unitId, alarmDetail.toString());
    }


    private Integer getAdUnitStatus(Long advertiserId, Long unitId, String accessToken) {
        JSONObject param = new JSONObject();
        param.put("advertiser_id", advertiserId);
        param.put("unit_id", unitId);
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.kuaiShouhttpPostRequest(postUrl + KuaishouConstant.AD_UNIT_LIST, param.toJSONString(), headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            log.info("accountId:{} 广告组数据信息为空！", advertiserId);
            return -1;
        }
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("查询广告组数据异常:{},accountId:{}", message, advertiserId);
            return -1;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.info("快手广告组数据为空=》accountId:{}", advertiserId);
            return -1;
        }
        //投放状态（操作结果）	1：投放中；2：暂停 3：删除
        return details.getJSONObject(0).getInteger("put_status");
    }


    @Autowired
    private RedisUtil redisUtil;

    @Override
    public void getVideoList(Long advertiserId, String token, String startDate, String endDate, int page) {
        log.info("短剧-开始获取所有视频信息 accountID:{},page:{}", advertiserId, page);
        List<Object> adAccountList = redisUtil.getList("com_kuaiShou_allVideo_list", null);
        String url = postUrl + KuaishouConstant.VIDEO_LIST;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", token);
        Map<String, Object> param = new HashMap<>();
        param.put("advertiser_id", advertiserId);
        Integer page_size = 500;
        param.put("page_size", page_size);
        param.put("page", page);
        if (!Check.isNull(startDate) && !Check.isNull(endDate)) {
            param.put("start_date", startDate);
            param.put("end_date", endDate);
        }
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            return;
        }
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("短剧-获取快手视频列表数据异常:{},accountId:{}", message, advertiserId);

            //请求过于频繁
            if (400001 == code) {
                log.info("短剧-定时任务-->>allVideoList<<<<----获取所有快手视频请求频繁,code:{},准备重试,accountId:{},错误消息:{}", code, advertiserId, message);
                //添加重试
                if (Check.isNull(adAccountList)) {
                    redisUtil.add("com_kuaiShou_allVideo_list", Arrays.asList(advertiserId), 1, TimeUnit.HOURS);
                } else {
                    adAccountList.add(advertiserId);
                    redisUtil.add("com_kuaiShou_allVideo_list", adAccountList, 1, TimeUnit.HOURS);
                }
            } else {
                if (adAccountList.contains(advertiserId)) {
                    adAccountList.remove(advertiserId);
                    redisUtil.add("com_kuaiShou_allVideo_list", advertiserId, 1, TimeUnit.HOURS);
                }
            }
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.info("快手视频列表信息为空=》accountId:{}", advertiserId);
            return;
        }
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        List<KuaishouVideoList> videoGetList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject detailJson = details.getJSONObject(i);
            if (Check.isNull(detailJson)) {
                continue;
            }
            KuaishouVideoList videoList = new KuaishouVideoList();
            String photo_id = detailJson.getString("photo_id");
            videoList.setAccountId(advertiserId);
            videoList.setCoverUrl(detailJson.getString("cover_url"));
            videoList.setPhotoId(photo_id);
            videoList.setPhotoName(detailJson.getString("photo_name"));
            videoList.setStatDate(detailJson.getDate("upload_time"));
            String signature = detailJson.getString("signature");
            videoList.setSignature(signature);
            Integer source = detailJson.getInteger("source");
            if (source == 2) {
                videoList.setChannelType(1);
            } else {
                videoList.setChannelType(0);
            }
            Integer new_status = detailJson.getInteger("new_status");
            if (new_status == 1) {
                videoList.setStatus(0);
            } else {
                videoList.setStatus(1);
            }
            videoList.setUrl(detailJson.getString("url"));
            Integer width = detailJson.getInteger("width");
            videoList.setWidth(width);
            Integer height = detailJson.getInteger("height");
            videoList.setHeight(height);
            Integer type = MaterialEnum.getTypeBySize(width, height);
            if (!Check.isNull(type)) {
                videoList.setMaterialType(type);
            }
            videoGetList.add(videoList);
        }
        if (!Check.isNull(videoGetList)) {
            baseMapper.replaceBatchVideoList(videoGetList);
        }
        if (toGet) {
            getVideoList(advertiserId, token, startDate, endDate, page + 1);
        } else {
            log.info("短剧-视频列表数据获取完成：accountId：{}", advertiserId);
        }
    }

    @Override
    public CompassBindAdvertisers getByAccountId(String accountId) {
        return baseMapper.getByAccountId(accountId);
    }

    @Override
    public List<Long> getAccountList() {
        return baseMapper.getAccountList();
    }
}
