package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.constant.KuaishouConstant;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAccountReportDaily;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouEffectNativeChartreportConversionChart;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouEffectNativeChartreportFunnel;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouEffectNativeChartreportSummary;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouAccountReportDailyMapper;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportDailyService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 快手-账户日报
 *
 * @author jeecg-boot 2021-09-07
 * @version V1.0
 */
@Slf4j
@Service
public class KuaishouAccountReportDailyServiceImpl extends ServiceImpl<KuaishouAccountReportDailyMapper, KuaishouAccountReportDaily> implements IKuaishouAccountReportDailyService {

    @Autowired
    private KuaishouAccountReportDailyMapper accountReportDailyMapper;

    @Value("${api.kuaishou.postUrl}")
    private String postUrl;

    @Override
    public void getAdvertiserReportDaily(Long advertiserId, String accessToken, String startDate, String endDate, int page) throws Exception {
        log.info("获取账户日报，账户id：{}，开始时间：{}，结束时间：{}", advertiserId, startDate, endDate);
        String url = postUrl + KuaishouConstant.ACCOUNT_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "DAILY");
        param.put("page_size", 2000);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        //   System.err.println(resultJson);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手广告主日报表异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手日报返回详情为空，accountId：{}", advertiserId);
            return;
        }
        List<KuaishouAccountReportDaily> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouAccountReportDaily reportDaily = KuaishouAccountReportDaily.getKuaishouAccountReportDailyInfo(returnJson);
            reportDaily.setAdvertiserId(advertiserId);
            addList.add(reportDaily);
        }
        accountReportDailyMapper.replaceBatch(addList);
        sendMessage(advertiserId, startDate, endDate, "daily");
        log.info("账户日报数据获取完成：accountId：{}", advertiserId);
    }

    @Override
    public void sendMessage(Long accountId, String startDate, String endDate, String date_type) {
        String url = KuaishouConstant.SEND_URL;
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", DateUtils.getDateInteger(startDate));
        param.put("end_date", DateUtils.getDateInteger(endDate));
        param.put("account_id", accountId);
        param.put("date_type", date_type);
        String result = HttpUtils.httpPostRequest(url, param, null);
        System.err.println(result);
    }

    @Override
    public void effectNativeChartReport(Long accountId, String token, String beginDate, String endDate) {
        String url = postUrl + KuaishouConstant.EFFECT_NATIVE_CHART_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", token);
        Map<String, Object> param = new HashMap<>();
        param.put("advertiser_id", accountId);
        JSONObject search_param = new JSONObject();

        search_param.put("report_start_day", DateUtils.timeToStamp(beginDate + " 00:00:00"));
        search_param.put("report_end_day", DateUtils.timeToStamp(endDate + " 23:59:59"));
        param.put("search_param", search_param);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
//        System.out.println("-------------------" + param);
//        System.out.println("");
//        System.err.println(resultJson);
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            String message = resultJson.getString("message");
            log.error("获取快手获取原生效果数据整体概览接口（数据概览、分日转化数据分布、转化漏斗）异常:{},accountId:{}", message, accountId);
            return;
        }
        JSONObject data = resultJson.getJSONObject("data");

        //数据概览
        JSONArray summary = data.getJSONArray("summary");
        if (!Check.isNull(summary)) {
            List<KuaishouEffectNativeChartreportSummary> sumList = new ArrayList<>();
            for (int i = 0; i < summary.size(); i++) {
                JSONObject sumObj = summary.getJSONObject(i);
                KuaishouEffectNativeChartreportSummary sum = JSONObject.parseObject(sumObj.toJSONString(), KuaishouEffectNativeChartreportSummary.class);
                sum.setStatDate(DateUtils.getDateLong(beginDate));
                sum.setAccountId(accountId);

                sumList.add(sum);
            }
            if (!Check.isNull(sumList)) {
                accountReportDailyMapper.replaceBatchSummary(sumList);
            }
        }

        //漏斗数据
        JSONArray funnel = data.getJSONArray("funnel");
        if (!Check.isNull(funnel)) {
            List<KuaishouEffectNativeChartreportFunnel> funList = new ArrayList<>();
            for (int i = 0; i < funnel.size(); i++) {
                JSONObject funObj = funnel.getJSONObject(i);
                KuaishouEffectNativeChartreportFunnel fun = JSONObject.parseObject(funObj.toJSONString(), KuaishouEffectNativeChartreportFunnel.class);
                fun.setStatDate(DateUtils.getDateLong(beginDate));
                fun.setAccountId(accountId);

                funList.add(fun);
            }
            if (!Check.isNull(funList)) {
                accountReportDailyMapper.replaceBatchFunnel(funList);
            }
        }

        //转化数分日柱状图
        JSONArray conversionChart = data.getJSONArray("conversion_chart");
        if (!Check.isNull(conversionChart)) {
            List<KuaishouEffectNativeChartreportConversionChart> chatList = new ArrayList<>();
            for (int i = 0; i < conversionChart.size(); i++) {
                JSONObject chatObj = conversionChart.getJSONObject(i);
                KuaishouEffectNativeChartreportConversionChart chat = JSONObject.parseObject(chatObj.toJSONString(), KuaishouEffectNativeChartreportConversionChart.class);
                chat.setDate(DateUtils.getDateLong(beginDate));
                chat.setAccountId(accountId);

                chatList.add(chat);
            }
            if (!Check.isNull(chatList)) {
                accountReportDailyMapper.replaceBatchConversionChart(chatList);
            }
        }
    }

    @Override
    public void liveComponentReport(Long accountId, String token, String beginDate, String endDate) {
        String url = postUrl + KuaishouConstant.LIVE_COMPONENT_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", token);
        Map<String, Object> param = new HashMap<>();
        param.put("advertiser_id", accountId);
//        param.put("temporal_granularity", "HOURLY");
        param.put("start_date", beginDate);
        param.put("end_date", endDate);
        param.put("page", 1);
        param.put("page_size", 2000);
        JSONObject search_param = new JSONObject();

        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        System.out.println("---直播间组件报表----------------" + param);
        System.out.println("");
        System.err.println(resultJson);
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            String message = resultJson.getString("message");
            log.error("直播间组件报表异常:{},accountId:{}", message, accountId);
            return;
        }
        JSONObject data = resultJson.getJSONObject("data");
        JSONArray details = data.getJSONArray("details");

      /*  if (!Check.isNull(details)) {
            for (int i = 0; i < details.size(); i++) {
                JSONObject jsonObject = details.getJSONObject(i);
                jsonObject.put("start_date", DateUtils.tempToDate(jsonObject.getLong("live_start_time"), "yyyy-MM-dd HH:mm:ss"));
                jsonObject.put("end_date", DateUtils.tempToDate(jsonObject.getLong("live_end_time"), "yyyy-MM-dd HH:mm:ss"));
            }

            System.out.println("-------------------直播间" + param);
            System.out.println("");
            System.err.println(details);

        }*/

    }


    @Override
    public void liveUserReport(Long accountId, String token, String beginDate, String endDate) {
        String url = postUrl + KuaishouConstant.LIVE_USER_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", token);
        Map<String, Object> param = new HashMap<>();
        param.put("advertiser_id", accountId);
//        param.put("temporal_granularity", "HOURLY");
        param.put("start_date", beginDate);
        param.put("end_date", endDate);
        param.put("page", 1);
        param.put("page_size", 2000);
        JSONObject search_param = new JSONObject();
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        System.out.println("-------------------直播间" + param);
        System.out.println("");
        System.err.println(resultJson);
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            String message = resultJson.getString("message");
            log.error("直播间报表异常:{},accountId:{}", message, accountId);
            return;
        }
        JSONObject data = resultJson.getJSONObject("data");
        JSONArray details = data.getJSONArray("details");
        if (!Check.isNull(details)) {
            for (int i = 0; i < details.size(); i++) {
                JSONObject jsonObject = details.getJSONObject(i);
                jsonObject.put("start_date", DateUtils.tempToDate(jsonObject.getLong("live_start_time"), "yyyy-MM-dd HH:mm:ss"));
                jsonObject.put("end_date", DateUtils.tempToDate(jsonObject.getLong("live_end_time"), "yyyy-MM-dd HH:mm:ss"));
                jsonObject.put("charge", jsonObject.getBigDecimal("ad_live_follow").multiply(jsonObject.getBigDecimal("ad_live_follow_cost")));
            }

            System.err.println("-------------------直播间整理数据\n" + details);

        }
    }
}
