package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.constant.KuaishouConstant;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouAccountReportHourly;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouAccountReportHourlyMapper;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportHourlyService;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 快手-账户时报
 *
 * @author jeecg-boot 2021-09-08
 * @version V1.0
 */
@Slf4j
@Service
public class KuaishouAccountReportHourlyServiceImpl extends ServiceImpl<KuaishouAccountReportHourlyMapper, KuaishouAccountReportHourly> implements IKuaishouAccountReportHourlyService {

    @Value("${api.kuaishou.postUrl}")
    private String postUrl;
    @Autowired
    private KuaishouAccountReportHourlyMapper accountReportHourlyMapper;
    @Autowired
    private IKuaishouAccountReportDailyService reportDailyService;


    @Override
    public void getAdvertiserReportHour(Long advertiserId, String accessToken, String startDate, String endDate, int page) {
        String url = postUrl + KuaishouConstant.ACCOUNT_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "HOURLY");
        param.put("page_size", 2000);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手广告主时报表异常:{},accountId:{}", message, advertiserId);
            log.error("入参：{}", param);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手时报返回详情为空，accountId：{}", advertiserId);
            return;
        }
        List<KuaishouAccountReportHourly> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouAccountReportHourly reportHourly = KuaishouAccountReportHourly.getKuaishouAccountReportHourInfo(returnJson);
            reportHourly.setAdvertiserId(advertiserId);
            addList.add(reportHourly);
        }
        accountReportHourlyMapper.replaceBatch(addList);
        log.info("账户时报数据获取完成：accountId：{}", advertiserId);
        reportDailyService.sendMessage(advertiserId, startDate, endDate, "hourly");
    }
}
