package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.constant.KuaishouConstant;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouMaterialVideoReportHourly;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouMaterialVideoReportHourlyMapper;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouMaterialVideoReportHourlyService;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 素材-时报
 *
 * @author jeecg-boot 2021-09-09
 * @version V1.0
 */
@Slf4j
@Service
public class KuaishouMaterialVideoReportHourlyServiceImpl extends ServiceImpl<KuaishouMaterialVideoReportHourlyMapper, KuaishouMaterialVideoReportHourly> implements IKuaishouMaterialVideoReportHourlyService {
    @Value("${api.kuaishou.postUrl}")
    private String postUrl;
    @Autowired
    private KuaishouMaterialVideoReportHourlyMapper materialVideoReportHourlyMapper;

    @Override
    public void getVideoReportHour(Long advertiserId, String accessToken, String startDate, String endDate, int page) {
        String url = postUrl + KuaishouConstant.MATERIAL_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "HOURLY");
        param.put("view_type", 5);
        Integer page_size = 2000;
        param.put("page_size", page_size);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        JSONArray details = resultJson.getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手视频时报返回详情为空，accountId：{}", advertiserId);
            return;
        }
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        List<KuaishouMaterialVideoReportHourly> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouMaterialVideoReportHourly reportHourly = new KuaishouMaterialVideoReportHourly();
            reportHourly.setAdvertiserId(advertiserId);
            reportHourly.setStatDate(DateUtils.getDateInteger(returnJson.getString("stat_date")));
            reportHourly.setHour(returnJson.getInteger("stat_hour"));
            reportHourly.setPhotoId(returnJson.getString("photo_id"));
            reportHourly.setAdScene(returnJson.getString("ad_scene"));
            reportHourly.setPhotoUrl(returnJson.getString("photo_url"));
            reportHourly.setCoverUrl(returnJson.getString("cover_url"));
            reportHourly.setImageToken(returnJson.getString("image_token"));
            reportHourly.setPhotoCaption("");
            reportHourly.setCharge(returnJson.getBigDecimal("charge"));
            reportHourly.setShow(returnJson.getLong("show"));
            reportHourly.setPhotoClick(returnJson.getLong("photo_click"));
            reportHourly.setAclick(returnJson.getLong("aclick"));
            reportHourly.setBclick(returnJson.getLong("bclick"));
            reportHourly.setPhotoClickRatio(returnJson.getBigDecimal("photo_click_ratio"));
            reportHourly.setPlay3sRatio(returnJson.getBigDecimal("play_3s_ratio"));
            reportHourly.setActionRatio(returnJson.getBigDecimal("action_ratio"));
            reportHourly.setImpression1kCost(returnJson.getBigDecimal("impression_1k_cost"));
            reportHourly.setPhotoClickCost(returnJson.getBigDecimal("photo_click_cost"));
            reportHourly.setActionCost(returnJson.getBigDecimal("action_cost"));
            reportHourly.setShare(returnJson.getLong("share"));
            reportHourly.setComment(returnJson.getLong("comment"));
            reportHourly.setLike(returnJson.getLong("like"));
            reportHourly.setFollow(returnJson.getLong("follow"));
            reportHourly.setCancelFollow(returnJson.getLong("cancel_follow"));
            reportHourly.setReport(returnJson.getLong("report"));
            reportHourly.setBlock(returnJson.getLong("block"));
            reportHourly.setNegative(returnJson.getLong("negative"));
            reportHourly.setSubmit(returnJson.getLong("submit"));
            reportHourly.setDownloadStarted(returnJson.getLong("download_started"));
            reportHourly.setDownloadCompleted(returnJson.getLong("download_completed"));
            reportHourly.setActivation(returnJson.getLong("activation"));
            reportHourly.setEventPayFirstDay(returnJson.getLong("event_pay_first_day"));
            reportHourly.setEventPayPurchaseAmountFirstDay(returnJson.getBigDecimal("event_pay_purchase_amount_first_day"));
            reportHourly.setEventPayFirstDayRoi(returnJson.getBigDecimal("event_pay_first_day_roi"));
            reportHourly.setEventPay(returnJson.getLong("event_pay"));
            reportHourly.setEventPayPurchaseAmount(returnJson.getBigDecimal("event_pay_purchase_amount"));
            reportHourly.setEventPayRoi(returnJson.getBigDecimal("event_pay_roi"));
            reportHourly.setEventRegister(returnJson.getLong("event_register"));
            reportHourly.setEventRegisterCost(returnJson.getBigDecimal("event_register_cost"));
            reportHourly.setEventRegisterRatio(returnJson.getBigDecimal("event_register_ratio"));
            reportHourly.setEventJinJianApp(returnJson.getLong("event_jin_jian_app"));
            reportHourly.setEventJinJianAppCost(returnJson.getBigDecimal("event_jin_jian_app_cost"));
            reportHourly.setEventCreditGrantApp(returnJson.getLong("event_credit_grant_app"));
            reportHourly.setEventCreditGrantAppCost(returnJson.getBigDecimal("event_credit_grant_app_cost"));
            reportHourly.setEventCreditGrantAppRatio(returnJson.getBigDecimal("event_credit_grant_app_ratio"));
            reportHourly.setEventOrderPaid(returnJson.getLong("event_order_paid"));
            reportHourly.setEventOrderPaidPurchaseAmount(returnJson.getBigDecimal("event_order_paid_purchase_amount"));
            reportHourly.setEventOrderPaidCost(returnJson.getBigDecimal("event_order_paid_cost"));
            reportHourly.setEventNextDayStay(returnJson.getLong("event_next_day_stay"));
            reportHourly.setEventNextDayStayCost(returnJson.getBigDecimal("event_next_day_stay_cost"));
            reportHourly.setEventNextDayStayRatio(returnJson.getBigDecimal("event_next_day_stay_ratio"));
            reportHourly.setFormCount(returnJson.getLong("form_count"));
            reportHourly.setFormCost(returnJson.getBigDecimal("form_cost"));
            reportHourly.setFormActionRatio(returnJson.getBigDecimal("form_action_ratio"));
            reportHourly.setEventJinJianLandingPage(returnJson.getLong("event_jin_jian_landing_page"));
            reportHourly.setEventJinJianLandingPageCost(returnJson.getBigDecimal("event_jin_jian_landing_page_cost"));
            reportHourly.setEventCreditGrantLandingPage(returnJson.getLong("event_credit_grant_landing_page"));
            reportHourly.setEventCreditGrantLandingPageCost(returnJson.getBigDecimal("event_credit_grant_landing_page_cost"));
            reportHourly.setEventCreditGrantLandingPageRatio(returnJson.getBigDecimal("event_credit_grant_landing_page_ratio"));
            reportHourly.setEventValidClues(returnJson.getLong("event_valid_clues"));
            reportHourly.setEventValidCluesCost(returnJson.getBigDecimal("event_valid_clues_cost"));
            reportHourly.setEventAddWechat(returnJson.getLong("event_add_wechat"));
            reportHourly.setEventAddWechatCost(returnJson.getBigDecimal("event_add_wechat_cost"));
            reportHourly.setEventAddWechatRatio(returnJson.getBigDecimal("event_add_wechat_ratio"));
            reportHourly.setEventGetThrough(returnJson.getLong("event_get_through"));
            reportHourly.setEventGetThroughCost(returnJson.getBigDecimal("event_get_through_cost"));
            reportHourly.setEventGetThroughRatio(returnJson.getBigDecimal("event_get_through_ratio"));
            reportHourly.setEventAppInvoked(returnJson.getLong("event_app_invoked"));
            reportHourly.setEventAppInvokedCost(returnJson.getBigDecimal("event_app_invoked_cost"));
            reportHourly.setEventAppInvokedRatio(returnJson.getBigDecimal("event_app_invoked_ratio"));
            reportHourly.setEventCreditGrantLandingRatio(returnJson.getBigDecimal("event_credit_grant_landing_ratio"));
            reportHourly.setPlay5sRatio(returnJson.getBigDecimal("play_5s_ratio"));
            reportHourly.setPlayEndRatio(returnJson.getBigDecimal("play_end_ratio"));
            reportHourly.setEventNewUserPay(returnJson.getBigDecimal("event_new_user_pay"));
            reportHourly.setEventNewUserPayCost(returnJson.getBigDecimal("event_new_user_pay_cost"));
            reportHourly.setEventNewUserPayRatio(returnJson.getBigDecimal("event_new_user_pay_ratio"));
            reportHourly.setClick1kCost(returnJson.getBigDecimal("click_1k_cost"));
            reportHourly.setAdProductCnt(returnJson.getLong("ad_product_cnt"));
            reportHourly.setEventGoodsView(returnJson.getLong("event_goods_view"));
            reportHourly.setMerchantRecoFans(returnJson.getLong("merchant_reco_fans"));
            reportHourly.setEventGoodsViewCost(returnJson.getBigDecimal("event_goods_view_cost"));
            reportHourly.setMerchantRecoFansCost(returnJson.getBigDecimal("merchant_reco_fans_cost"));
            reportHourly.setEventOrderAmountRoi(returnJson.getBigDecimal("event_order_amount_roi"));
            reportHourly.setEventButtonClick(returnJson.getLong("event_button_click"));
            reportHourly.setEventButtonClickCost(returnJson.getBigDecimal("event_button_click_cost"));
            reportHourly.setEventButtonClickRatio(returnJson.getBigDecimal("event_button_click_ratio"));
            reportHourly.setEventOrderPaidRoi(returnJson.getBigDecimal("event_order_paid_roi"));
            reportHourly.setEventNewUserJinjianApp(returnJson.getLong("event_new_user_jinjian_app"));
            reportHourly.setEventNewUserJinjianAppCost(returnJson.getBigDecimal("event_new_user_jinjian_app_cost"));
            reportHourly.setEventNewUserJinjianAppRoi(returnJson.getBigDecimal("event_new_user_jinjian_app_roi"));
            reportHourly.setEventNewUserCreditGrantApp(returnJson.getLong("event_new_user_credit_grant_app"));
            reportHourly.setEventNewUserCreditGrantAppCost(returnJson.getBigDecimal("event_new_user_credit_grant_app_cost"));
            reportHourly.setEventNewUserCreditGrantAppRoi(returnJson.getBigDecimal("event_new_user_credit_grant_app_roi"));
            reportHourly.setEventNewUserJinjianPage(returnJson.getLong("event_new_user_jinjian_page"));
            reportHourly.setEventNewUserJinjianPageCost(returnJson.getBigDecimal("event_new_user_jinjian_page_cost"));
            reportHourly.setEventNewUserJinjianPageRoi(returnJson.getBigDecimal("event_new_user_jinjian_page_roi"));
            reportHourly.setEventNewUserCreditGrantPage(returnJson.getLong("event_new_user_credit_grant_page"));
            reportHourly.setEventNewUserCreditGrantPageCost(returnJson.getBigDecimal("event_new_user_credit_grant_page_cost"));
            reportHourly.setEventNewUserCreditGrantPageRoi(returnJson.getBigDecimal("event_new_user_credit_grant_page_roi"));
            reportHourly.setEventAppointForm(returnJson.getLong("event_appoint_form"));
            reportHourly.setEventAppointFormCost(returnJson.getBigDecimal("event_appoint_form_cost"));
            reportHourly.setEventAppointFormRatio(returnJson.getBigDecimal("event_appoint_form_ratio"));
            reportHourly.setEventAppointJumpClick(returnJson.getLong("event_appoint_jump_click"));
            reportHourly.setEventAppointJumpClickCost(returnJson.getBigDecimal("event_appoint_jump_click_cost"));
            reportHourly.setEventAppointJumpClickRatio(returnJson.getBigDecimal("event_appoint_jump_click_ratio"));
            reportHourly.setUnionEventPayPurchaseAmount7d(returnJson.getLong("union_event_pay_purchase_amount_7d"));
            reportHourly.setUnionEventPayPurchaseAmount7dRoi(returnJson.getBigDecimal("union_event_pay_purchase_amount_7d_roi"));
            reportHourly.setPlacementType(returnJson.getString("placement_type"));
            reportHourly.setEventOrderSuccessed(returnJson.getLong("event_order_successed"));
            reportHourly.setAdPhotoPlayed10sRatio(returnJson.getBigDecimal("ad_photo_played_10s_ratio"));
            reportHourly.setKeyActionCost(returnJson.getBigDecimal("key_action_cost"));
            reportHourly.setEventAddShoppingCartCost(returnJson.getBigDecimal("event_add_shopping_cart_cost"));
            reportHourly.setEventAdWatchTimesRatio(returnJson.getBigDecimal("event_ad_watch_times_ratio"));
            reportHourly.setEventOutboundCallCost(returnJson.getBigDecimal("event_outbound_call_cost"));
            reportHourly.setEventOutboundCall(returnJson.getLong("event_outbound_call"));
            reportHourly.setActionNewRatio(returnJson.getBigDecimal("action_new_ratio"));
            reportHourly.setEventWatchAppAd(returnJson.getLong("event_watch_app_ad"));
            reportHourly.setEventMultiConversionCost(returnJson.getBigDecimal("event_multi_conversion_cost"));
            reportHourly.setEventPhoneCardActivate(returnJson.getLong("event_phone_card_activate"));
            reportHourly.setAdPhotoPlayed75percentRatio(returnJson.getBigDecimal("ad_photo_played_75percent_ratio"));
            // reportHourly.setKeyActionRatio(returnJson.getBigDecimal("key_action_ratio"));
            reportHourly.setEventAdWatchTimesCost(returnJson.getBigDecimal("event_ad_watch_times_cost"));
            reportHourly.setEventAddShoppingCart(returnJson.getLong("event_add_shopping_cart"));
            reportHourly.setKeyAction(returnJson.getLong("key_action"));
            reportHourly.setEventMultiConversion(returnJson.getLong("event_multi_conversion"));
            reportHourly.setEventWechatConnected(returnJson.getLong("event_wechat_connected"));
            reportHourly.setEventDspGiftForm(returnJson.getLong("event_dsp_gift_form"));
            reportHourly.setEventIntentionConfirmed(returnJson.getLong("event_intention_confirmed"));
            reportHourly.setEventPhoneGetThrough(returnJson.getLong("event_phone_get_through"));
            reportHourly.setEventMultiConversionRatio(returnJson.getBigDecimal("event_multi_conversion_ratio"));
            reportHourly.setEventMeasurementHouse(returnJson.getBigDecimal("event_measurement_house"));
            reportHourly.setAdPhotoPlayed2sRatio(returnJson.getBigDecimal("ad_photo_played_2s_ratio"));
            reportHourly.setEventOutboundCallRatio(returnJson.getBigDecimal("event_outbound_call_ratio"));
            reportHourly.setEventAdWatchTimes(returnJson.getLong("event_ad_watch_times"));
            addList.add(reportHourly);
        }
        materialVideoReportHourlyMapper.replaceBatch(addList);
        if (toGet) {
            getVideoReportHour(advertiserId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("视频时报数据获取完成：accountId：{}", advertiserId);
        }
    }
}
