package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.constant.KuaishouConstant;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouProgramCreativeReportDaily;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouProgramCreativeReportDailyMapper;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouProgramCreativeReportDailyService;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 程序化创意-日报
 *
 * @author jeecg-boot 2021-09-09
 * @version V1.0
 */
@Slf4j
@Service
public class KuaishouProgramCreativeReportDailyServiceImpl extends ServiceImpl<KuaishouProgramCreativeReportDailyMapper, KuaishouProgramCreativeReportDaily> implements IKuaishouProgramCreativeReportDailyService {
    @Value("${api.kuaishou.postUrl}")
    private String postUrl;
    @Autowired
    private KuaishouProgramCreativeReportDailyMapper programCreativeReportDailyMapper;

    @Override
    public void getProgramCreativeReportDaily(Long advertiserId, String accessToken, String startDate, String endDate, int page) {
        String url = postUrl + KuaishouConstant.PROGRAM_CREATIVE_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "DAILY");
        Integer page_size = 2000;
        param.put("page_size", page_size);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手广告程序化创意日报表异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手程序化创意日报返回详情为空，accountId：{}", advertiserId);
            return;
        }
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        List<KuaishouProgramCreativeReportDaily> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouProgramCreativeReportDaily reportDaily = new KuaishouProgramCreativeReportDaily();
            reportDaily.setAdvertiserId(advertiserId);
            reportDaily.setStatDate(DateUtils.getDateInteger(returnJson.getString("stat_date")));
            reportDaily.setCampaignId(returnJson.getLong("campaign_id"));
            reportDaily.setCampaignName(returnJson.getString("campaign_name"));
            reportDaily.setUnitId(returnJson.getLong("unit_id"));
            reportDaily.setUnitName(returnJson.getString("unit_name"));
            reportDaily.setCreativeId(returnJson.getLong("creative_id"));
            reportDaily.setPhotoId(returnJson.getString("photo_id"));
            reportDaily.setPhotoUrl(returnJson.getString("photo_url"));
            reportDaily.setCoverUrl(returnJson.getString("cover_url"));
            reportDaily.setImageToken(returnJson.getString("image_token"));
            reportDaily.setDescription(returnJson.getString("description"));
            reportDaily.setCharge(returnJson.getBigDecimal("charge"));
            reportDaily.setShow(returnJson.getLong("show"));
            reportDaily.setPhotoClick(returnJson.getLong("photo_click"));
            reportDaily.setAclick(returnJson.getLong("aclick"));
            reportDaily.setBclick(returnJson.getLong("bclick"));
            reportDaily.setPhotoClickRatio(returnJson.getBigDecimal("photo_click_ratio"));
            reportDaily.setPlay3sRatio(returnJson.getBigDecimal("play_3s_ratio"));
            reportDaily.setActionRatio(returnJson.getBigDecimal("action_ratio"));
            reportDaily.setImpression1kCost(returnJson.getBigDecimal("impression_1k_cost"));
            reportDaily.setPhotoClickCost(returnJson.getBigDecimal("photo_click_cost"));
            reportDaily.setActionCost(returnJson.getBigDecimal("action_cost"));
            reportDaily.setShare(returnJson.getLong("share"));
            reportDaily.setComment(returnJson.getLong("comment"));
            reportDaily.setLike(returnJson.getLong("like"));
            reportDaily.setFollow(returnJson.getLong("follow"));
            reportDaily.setCancelFollow(returnJson.getLong("cancel_follow"));
            reportDaily.setReport(returnJson.getLong("report"));
            reportDaily.setBlock(returnJson.getLong("block"));
            reportDaily.setNegative(returnJson.getLong("negative"));
            reportDaily.setSubmit(returnJson.getLong("submit"));
            reportDaily.setDownloadStarted(returnJson.getLong("download_started"));
            reportDaily.setDownloadCompleted(returnJson.getLong("download_completed"));
            reportDaily.setActivation(returnJson.getLong("activation"));
            reportDaily.setEventPayFirstDay(returnJson.getLong("event_pay_first_day"));
            reportDaily.setEventPayPurchaseAmountFirstDay(returnJson.getBigDecimal("event_pay_purchase_amount_first_day"));
            reportDaily.setEventPayFirstDayRoi(returnJson.getBigDecimal("event_pay_first_day_roi"));
            reportDaily.setEventPay(returnJson.getLong("event_pay"));
            reportDaily.setEventPayPurchaseAmount(returnJson.getBigDecimal("event_pay_purchase_amount"));
            reportDaily.setEventPayRoi(returnJson.getBigDecimal("event_pay_roi"));
            reportDaily.setEventRegister(returnJson.getLong("event_register"));
            reportDaily.setEventRegisterCost(returnJson.getBigDecimal("event_register_cost"));
            reportDaily.setEventRegisterRatio(returnJson.getBigDecimal("event_register_ratio"));
            reportDaily.setEventJinJianApp(returnJson.getLong("event_jin_jian_app"));
            reportDaily.setEventJinJianAppCost(returnJson.getBigDecimal("event_jin_jian_app_cost"));
            reportDaily.setEventCreditGrantApp(returnJson.getLong("event_credit_grant_app"));
            reportDaily.setEventCreditGrantAppCost(returnJson.getBigDecimal("event_credit_grant_app_cost"));
            reportDaily.setEventCreditGrantAppRatio(returnJson.getBigDecimal("event_credit_grant_app_ratio"));
            reportDaily.setEventOrderPaid(returnJson.getLong("event_order_paid"));
            reportDaily.setEventOrderPaidPurchaseAmount(returnJson.getBigDecimal("event_order_paid_purchase_amount"));
            reportDaily.setEventOrderPaidCost(returnJson.getBigDecimal("event_order_paid_cost"));
            reportDaily.setEventNextDayStay(returnJson.getLong("event_next_day_stay"));
            reportDaily.setEventNextDayStayCost(returnJson.getBigDecimal("event_next_day_stay_cost"));
            reportDaily.setEventNextDayStayRatio(returnJson.getBigDecimal("event_next_day_stay_ratio"));
            reportDaily.setFormCount(returnJson.getLong("form_count"));
            reportDaily.setFormCost(returnJson.getBigDecimal("form_cost"));
            reportDaily.setFormActionRatio(returnJson.getBigDecimal("form_action_ratio"));
            reportDaily.setEventJinJianLandingPage(returnJson.getLong("event_jin_jian_landing_page"));
            reportDaily.setEventJinJianLandingPageCost(returnJson.getBigDecimal("event_jin_jian_landing_page_cost"));
            reportDaily.setEventCreditGrantLandingPage(returnJson.getLong("event_credit_grant_landing_page"));
            reportDaily.setEventCreditGrantLandingPageCost(returnJson.getBigDecimal("event_credit_grant_landing_page_cost"));
            reportDaily.setEventCreditGrantLandingPageRatio(returnJson.getBigDecimal("event_credit_grant_landing_page_ratio"));
            reportDaily.setEventValidClues(returnJson.getLong("event_valid_clues"));
            reportDaily.setEventValidCluesCost(returnJson.getBigDecimal("event_valid_clues_cost"));
            reportDaily.setEventAddWechat(returnJson.getLong("event_add_wechat"));
            reportDaily.setEventAddWechatCost(returnJson.getBigDecimal("event_add_wechat_cost"));
            reportDaily.setEventAddWechatRatio(returnJson.getBigDecimal("event_add_wechat_ratio"));
            reportDaily.setEventGetThrough(returnJson.getLong("event_get_through"));
            reportDaily.setEventGetThroughCost(returnJson.getBigDecimal("event_get_through_cost"));
            reportDaily.setEventGetThroughRatio(returnJson.getBigDecimal("event_get_through_ratio"));
            reportDaily.setEventAppInvoked(returnJson.getLong("event_app_invoked"));
            reportDaily.setEventAppInvokedCost(returnJson.getBigDecimal("event_app_invoked_cost"));
            reportDaily.setEventAppInvokedRatio(returnJson.getBigDecimal("event_app_invoked_ratio"));
            reportDaily.setEventCreditGrantLandingRatio(returnJson.getBigDecimal("event_credit_grant_landing_ratio"));
            reportDaily.setPlay5sRatio(returnJson.getBigDecimal("play_5s_ratio"));
            reportDaily.setPlayEndRatio(returnJson.getBigDecimal("play_end_ratio"));
            reportDaily.setEventNewUserPay(returnJson.getBigDecimal("event_new_user_pay"));
            reportDaily.setEventNewUserPayCost(returnJson.getBigDecimal("event_new_user_pay_cost"));
            reportDaily.setEventNewUserPayRatio(returnJson.getBigDecimal("event_new_user_pay_ratio"));
            reportDaily.setClick1kCost(returnJson.getBigDecimal("click_1k_cost"));
            reportDaily.setAdProductCnt(returnJson.getLong("ad_product_cnt"));
            reportDaily.setEventGoodsView(returnJson.getLong("event_goods_view"));
            reportDaily.setMerchantRecoFans(returnJson.getLong("merchant_reco_fans"));
            reportDaily.setEventGoodsViewCost(returnJson.getBigDecimal("event_goods_view_cost"));
            reportDaily.setMerchantRecoFansCost(returnJson.getBigDecimal("merchant_reco_fans_cost"));
            reportDaily.setEventOrderAmountRoi(returnJson.getBigDecimal("event_order_amount_roi"));
            reportDaily.setEventButtonClick(returnJson.getLong("event_button_click"));
            reportDaily.setEventButtonClickCost(returnJson.getBigDecimal("event_button_click_cost"));
            reportDaily.setEventButtonClickRatio(returnJson.getBigDecimal("event_button_click_ratio"));
            reportDaily.setEventOrderPaidRoi(returnJson.getBigDecimal("event_order_paid_roi"));
            reportDaily.setEventNewUserJinjianApp(returnJson.getLong("event_new_user_jinjian_app"));
            reportDaily.setEventNewUserJinjianAppCost(returnJson.getBigDecimal("event_new_user_jinjian_app_cost"));
            reportDaily.setEventNewUserJinjianAppRoi(returnJson.getBigDecimal("event_new_user_jinjian_app_roi"));
            reportDaily.setEventNewUserCreditGrantApp(returnJson.getLong("event_new_user_credit_grant_app"));
            reportDaily.setEventNewUserCreditGrantAppCost(returnJson.getBigDecimal("event_new_user_credit_grant_app_cost"));
            reportDaily.setEventNewUserCreditGrantAppRoi(returnJson.getBigDecimal("event_new_user_credit_grant_app_roi"));
            reportDaily.setEventNewUserJinjianPage(returnJson.getLong("event_new_user_jinjian_page"));
            reportDaily.setEventNewUserJinjianPageCost(returnJson.getBigDecimal("event_new_user_jinjian_page_cost"));
            reportDaily.setEventNewUserJinjianPageRoi(returnJson.getBigDecimal("event_new_user_jinjian_page_roi"));
            reportDaily.setEventNewUserCreditGrantPage(returnJson.getLong("event_new_user_credit_grant_page"));
            reportDaily.setEventNewUserCreditGrantPageCost(returnJson.getBigDecimal("event_new_user_credit_grant_page_cost"));
            reportDaily.setEventNewUserCreditGrantPageRoi(returnJson.getBigDecimal("event_new_user_credit_grant_page_roi"));
            reportDaily.setEventAppointForm(returnJson.getLong("event_appoint_form"));
            reportDaily.setEventAppointFormCost(returnJson.getBigDecimal("event_appoint_form_cost"));
            reportDaily.setEventAppointFormRatio(returnJson.getBigDecimal("event_appoint_form_ratio"));
            reportDaily.setEventAppointJumpClick(returnJson.getLong("event_appoint_jump_click"));
            reportDaily.setEventAppointJumpClickCost(returnJson.getBigDecimal("event_appoint_jump_click_cost"));
            reportDaily.setEventAppointJumpClickRatio(returnJson.getBigDecimal("event_appoint_jump_click_ratio"));
            reportDaily.setUnionEventPayPurchaseAmount7d(returnJson.getLong("union_event_pay_purchase_amount_7d"));
            reportDaily.setUnionEventPayPurchaseAmount7dRoi(returnJson.getBigDecimal("union_event_pay_purchase_amount_7d_roi"));
            reportDaily.setPlacementType(returnJson.getString("placement_type"));
            reportDaily.setEventOrderSuccessed(returnJson.getLong("event_order_successed"));
            reportDaily.setAdPhotoPlayed10sRatio(returnJson.getBigDecimal("ad_photo_played_10s_ratio"));
            reportDaily.setKeyActionCost(returnJson.getBigDecimal("key_action_cost"));
            reportDaily.setEventAddShoppingCartCost(returnJson.getBigDecimal("event_add_shopping_cart_cost"));
            reportDaily.setEventAdWatchTimesRatio(returnJson.getBigDecimal("event_ad_watch_times_ratio"));
            reportDaily.setEventOutboundCallCost(returnJson.getBigDecimal("event_outbound_call_cost"));
            reportDaily.setEventOutboundCall(returnJson.getLong("event_outbound_call"));
            reportDaily.setActionNewRatio(returnJson.getBigDecimal("action_new_ratio"));
            reportDaily.setEventWatchAppAd(returnJson.getLong("event_watch_app_ad"));
            reportDaily.setEventMultiConversionCost(returnJson.getBigDecimal("event_multi_conversion_cost"));
            reportDaily.setEventPhoneCardActivate(returnJson.getLong("event_phone_card_activate"));
            reportDaily.setAdPhotoPlayed75percentRatio(returnJson.getBigDecimal("ad_photo_played_75percent_ratio"));
     //       reportDaily.setKeyActionRatio(returnJson.getBigDecimal("key_action_ratio"));
            reportDaily.setEventAdWatchTimesCost(returnJson.getBigDecimal("event_ad_watch_times_cost"));
            reportDaily.setEventAddShoppingCart(returnJson.getLong("event_add_shopping_cart"));
            reportDaily.setKeyAction(returnJson.getLong("key_action"));
            reportDaily.setEventMultiConversion(returnJson.getLong("event_multi_conversion"));
            reportDaily.setEventWechatConnected(returnJson.getLong("event_wechat_connected"));
            reportDaily.setEventDspGiftForm(returnJson.getLong("event_dsp_gift_form"));
            reportDaily.setEventIntentionConfirmed(returnJson.getLong("event_intention_confirmed"));
            reportDaily.setEventPhoneGetThrough(returnJson.getLong("event_phone_get_through"));
            reportDaily.setEventMultiConversionRatio(returnJson.getBigDecimal("event_multi_conversion_ratio"));
            reportDaily.setEventMeasurementHouse(returnJson.getBigDecimal("event_measurement_house"));
            reportDaily.setAdPhotoPlayed2sRatio(returnJson.getBigDecimal("ad_photo_played_2s_ratio"));
            reportDaily.setEventOutboundCallRatio(returnJson.getBigDecimal("event_outbound_call_ratio"));
            reportDaily.setEventAdWatchTimes(returnJson.getLong("event_ad_watch_times"));
            addList.add(reportDaily);
        }
        programCreativeReportDailyMapper.replaceBatch(addList);
        if (toGet) {
            getProgramCreativeReportDaily(advertiserId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("程序化创意日报数据获取完成：accountId：{}", advertiserId);
        }
    }
}
