package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.constant.KuaishouConstant;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouUnitAudienceAgeReportDaily;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouUnitAudienceCityReportDaily;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouUnitAudienceGenderReportDaily;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouUnitAudienceProvinceReportDaily;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouUnitAudienceAgeReportDailyMapper;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouUnitAudienceCityReportDailyMapper;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouUnitAudienceGenderReportDailyMapper;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouUnitAudienceProvinceReportDailyMapper;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouUnitAudienceAgeReportDailyService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 人群年龄报表
 *
 * @author jeecg-boot
 * 2022-02-15
 * @version V1.0
 */
@Slf4j
@Service
public class KuaishouUnitAudienceAgeReportDailyServiceImpl extends ServiceImpl<KuaishouUnitAudienceAgeReportDailyMapper, KuaishouUnitAudienceAgeReportDaily> implements IKuaishouUnitAudienceAgeReportDailyService {
    @Value("${api.kuaishou.postUrl}")
    private String postUrl;

    @Resource
    private KuaishouUnitAudienceAgeReportDailyMapper ageMapper;

    @Resource
    private KuaishouUnitAudienceGenderReportDailyMapper genderMapper;

    @Resource
    private KuaishouUnitAudienceCityReportDailyMapper cityMapper;

    @Resource
    private KuaishouUnitAudienceProvinceReportDailyMapper provinceMapper;


    private static ExecutorService unitExecutorService = Executors.newFixedThreadPool(5);

    @Override
    public void standardCrowdAnalysisReport(Long accountId, String token, String date, Long campaignId, List<Long> unitIds) {

        //人群报表类型
        List<String> reportDimsProvince = Arrays.asList("province");
        List<String> reportDimsCity = Arrays.asList("city");
        List<String> reportDimsGender = Arrays.asList("gender");
        List<String> reportAge = Arrays.asList("ageSegment");
        List<Long> campaignIds = Arrays.asList(campaignId);
        if (!Check.isNull(unitIds)) {
            unitExecutorService.submit(
                    new Runnable() {
                        @Override
                        public void run() {
                            crowdAnalysisReport(accountId, token, date, campaignIds, unitIds, reportDimsProvince, 1);
                            crowdAnalysisReport(accountId, token, date, campaignIds, unitIds, reportDimsCity, 1);
                            crowdAnalysisReport(accountId, token, date, campaignIds, unitIds, reportDimsGender, 1);
                            crowdAnalysisReport(accountId, token, date, campaignIds, unitIds, reportAge, 1);
                        }
                    });
        }


    }


    @Override
    public void crowdAnalysisReport(Long accountId, String token, String date, List<Long> campaignIds, List<Long> unitIds, List<String> reportDims, int page) {
        JSONObject requestJson = new JSONObject();
        try {
            Integer pageSize = 2000;
            Map<String, String> headers = new HashMap<>();
            headers.put("Access-Token", token);
            headers.put("Content-Type", "application/json");
            requestJson.put("advertiser_id", accountId);
            requestJson.put("start_date", date);
            requestJson.put("end_date", date);
            requestJson.put("report_dims", reportDims);
            requestJson.put("page", page);
            requestJson.put("page_size", pageSize);
            requestJson.put("detailed", true);
            requestJson.put("campaign_ids", campaignIds);
            JSONArray units = new JSONArray();
            requestJson.put("unit_ids", unitIds);
            String result = HttpUtils.httpPostRequest(postUrl + KuaishouConstant.AUDIENCE_REPORT, requestJson, headers);
            Thread.sleep(500L);
            JSONObject resultJson = JSONObject.parseObject(result);
            if (!Check.isNull(resultJson)) {
                Integer code = resultJson.getInteger("code");
                if (code == 0) {
                    String type = reportDims.get(0);
                    JSONObject data = resultJson.getJSONObject("data");
                    JSONArray details = data.getJSONArray("details");
                    int pageLength = details.size();
                    if (Check.isNull(details) || details.isEmpty()) {
                        log.error("------快手组级人群报表返回为空，accountId：{}，结果---> {}", accountId,resultJson.toJSONString());
                        return;
                    }
                    List<KuaishouUnitAudienceProvinceReportDaily> provinceList = new ArrayList<>();
                    List<KuaishouUnitAudienceCityReportDaily> cityList = new ArrayList<>();
                    List<KuaishouUnitAudienceGenderReportDaily> genderList = new ArrayList<>();
                    List<KuaishouUnitAudienceAgeReportDaily> ageList = new ArrayList<>();
                    //组级
                    for (int i = 0; i < pageLength; i++) {
                        JSONObject obj = details.getJSONObject(i);
                        KuaishouUnitAudienceProvinceReportDaily entity = JSONObject.toJavaObject(obj, KuaishouUnitAudienceProvinceReportDaily.class);
                        entity.setAccountId(accountId);
                        entity.setCampaignId(campaignIds.get(0));
                        entity.setUnitId(obj.getLong("unit_id"));
                        entity.setStatDate(DateUtils.getDateInteger(obj.getString("stat_date")) + "");
                        if ("province".equals(type)) {
                            provinceList.add(entity);
                        } else if ("city".equals(type)) {
                            KuaishouUnitAudienceCityReportDaily cityEntity = new KuaishouUnitAudienceCityReportDaily();
                            BeanUtils.copyProperties(entity, cityEntity);
                            cityList.add(cityEntity);
                        } else if ("gender".equals(type)) {
                            KuaishouUnitAudienceGenderReportDaily genderEntity = new KuaishouUnitAudienceGenderReportDaily();
                            BeanUtils.copyProperties(entity, genderEntity);
                            genderEntity.setGender(obj.getString("gender"));
                            genderList.add(genderEntity);
                        } else if ("ageSegment".equals(type)) {
                            KuaishouUnitAudienceAgeReportDaily ageEntity = new KuaishouUnitAudienceAgeReportDaily();
                            BeanUtils.copyProperties(entity, ageEntity);
                            ageEntity.setAgeSegment(obj.getString("age_segment"));
                            ageList.add(ageEntity);
                        }
                    }
                    if ("province".equals(type)) {
                        provinceMapper.replaceBatch(provinceList);
                    } else if ("city".equals(type)) {
                        cityMapper.replaceBatch(cityList);
                    } else if ("gender".equals(type)) {
                        genderMapper.replaceBatch(genderList);
                    } else if ("ageSegment".equals(type)) {
                        ageMapper.replaceBatch(ageList);
                    }

                    if (pageLength == pageSize) {
                        crowdAnalysisReport(accountId, token, date, campaignIds, unitIds, reportDims, page + 1);
                    }

                } else {
                    if (code == 500000 || code == 400001) {
                        Thread.sleep(5000);
                        crowdAnalysisReport(accountId, token, date, campaignIds, unitIds, reportDims, page);
                    } else {
                        log.error("快手组级人群报表查询失败：入参：{}，返回信息：{}", requestJson.toJSONString(), resultJson.toJSONString());
                    }
                }
            }
        } catch (Exception e) {
            log.error("插入人群分析报表数据异常，入参：{}", requestJson.toJSONString(), e);
        }

    }

    @Override
    public List<Long> selectAccountIdsByTime(String date) {
        return ageMapper.selectAccountIdsByTime(date);
    }

    @Override
    public List<JSONObject> selectPlanIdsByAccountIdAndTime(Long accountId, String date) {
        return ageMapper.selectPlanIdsByAccountIdAndTime(accountId, date);
    }

    @Override
    public List<Long> selectUnitIdsByAccountIdAndTime(Long accountId, Long campaignId, String date) {
        return ageMapper.selectUnitIdsByAccountIdAndTime(accountId, campaignId, date);
    }
}
