package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.constant.KuaishouConstant;
import cn.com.ctop.job.kuaishou.data.entity.KuaishouUnitReportHourly;
import cn.com.ctop.job.kuaishou.data.mapper.KuaishouUnitReportHourlyMapper;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouUnitReportHourlyService;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 组-时报
 *
 * @author jeecg-boot 2021-09-09
 * @version V1.0
 */
@Slf4j
@Service
public class KuaishouUnitReportHourlyServiceImpl extends ServiceImpl<KuaishouUnitReportHourlyMapper, KuaishouUnitReportHourly> implements IKuaishouUnitReportHourlyService {
    @Value("${api.kuaishou.postUrl}")
    private String postUrl;
    @Autowired
    private KuaishouUnitReportHourlyMapper unitReportHourlyMapper;

    @Override
    public void getUnitReportHour(Long advertiserId, String accessToken, String startDate, String endDate, int page) {
        String url = postUrl + KuaishouConstant.UNIT_REPORT;
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", startDate);
        param.put("end_date", endDate);
        param.put("advertiser_id", advertiserId);
        param.put("temporal_granularity", "HOURLY");
        Integer page_size = 2000;
        param.put("page_size", page_size);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手广告组时报表异常:{},accountId:{}", message, advertiserId);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手组时报返回详情为空，accountId：{}", advertiserId);
            return;
        }
        Boolean toGet = true;
        if (details.size() < page_size) {
            toGet = false;
        }
        List<KuaishouUnitReportHourly> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject returnJson = details.getJSONObject(i);
            KuaishouUnitReportHourly reportHour = new KuaishouUnitReportHourly();
            reportHour.setAdvertiserId(advertiserId);
            reportHour.setStatDate(DateUtils.getDateInteger(returnJson.getString("stat_date")));
            reportHour.setHour(returnJson.getInteger("stat_hour"));
            reportHour.setCampaignId(returnJson.getLong("campaign_id"));
            reportHour.setCampaignName(returnJson.getString("campaign_name"));
            reportHour.setUnitId(returnJson.getLong("unit_id"));
            reportHour.setUnitName(returnJson.getString("unit_name"));
            reportHour.setCharge(returnJson.getBigDecimal("charge"));
            reportHour.setShow(returnJson.getLong("show"));
            reportHour.setPhotoClick(returnJson.getLong("photo_click"));
            reportHour.setAclick(returnJson.getLong("aclick"));
            reportHour.setBclick(returnJson.getLong("bclick"));
            reportHour.setPhotoClickRatio(returnJson.getBigDecimal("photo_click_ratio"));
            reportHour.setPlay3sRatio(returnJson.getBigDecimal("play_3s_ratio"));
            reportHour.setActionRatio(returnJson.getBigDecimal("action_ratio"));
            reportHour.setImpression1kCost(returnJson.getBigDecimal("impression_1k_cost"));
            reportHour.setPhotoClickCost(returnJson.getBigDecimal("photo_click_cost"));
            reportHour.setActionCost(returnJson.getBigDecimal("action_cost"));
            reportHour.setShare(returnJson.getLong("share"));
            reportHour.setComment(returnJson.getLong("comment"));
            reportHour.setLike(returnJson.getLong("like"));
            reportHour.setFollow(returnJson.getLong("follow"));
            reportHour.setCancelFollow(returnJson.getLong("cancel_follow"));
            reportHour.setReport(returnJson.getLong("report"));
            reportHour.setBlock(returnJson.getLong("block"));
            reportHour.setNegative(returnJson.getLong("negative"));
            reportHour.setSubmit(returnJson.getLong("submit"));
            reportHour.setDownloadStarted(returnJson.getLong("download_started"));
            reportHour.setDownloadCompleted(returnJson.getLong("download_completed"));
            reportHour.setActivation(returnJson.getLong("activation"));
            reportHour.setEventPayFirstDay(returnJson.getLong("event_pay_first_day"));
            reportHour.setEventPayPurchaseAmountFirstDay(returnJson.getBigDecimal("event_pay_purchase_amount_first_day"));
            reportHour.setEventPayFirstDayRoi(returnJson.getBigDecimal("event_pay_first_day_roi"));
            reportHour.setEventPay(returnJson.getLong("event_pay"));
            reportHour.setEventPayPurchaseAmount(returnJson.getBigDecimal("event_pay_purchase_amount"));
            reportHour.setEventPayRoi(returnJson.getBigDecimal("event_pay_roi"));
            reportHour.setEventRegister(returnJson.getLong("event_register"));
            reportHour.setEventRegisterCost(returnJson.getBigDecimal("event_register_cost"));
            reportHour.setEventRegisterRatio(returnJson.getBigDecimal("event_register_ratio"));
            reportHour.setEventJinJianApp(returnJson.getLong("event_jin_jian_app"));
            reportHour.setEventJinJianAppCost(returnJson.getBigDecimal("event_jin_jian_app_cost"));
            reportHour.setEventCreditGrantApp(returnJson.getLong("event_credit_grant_app"));
            reportHour.setEventCreditGrantAppCost(returnJson.getBigDecimal("event_credit_grant_app_cost"));
            reportHour.setEventCreditGrantAppRatio(returnJson.getBigDecimal("event_credit_grant_app_ratio"));
            reportHour.setEventOrderPaid(returnJson.getLong("event_order_paid"));
            reportHour.setEventOrderPaidPurchaseAmount(returnJson.getBigDecimal("event_order_paid_purchase_amount"));
            reportHour.setEventOrderPaidCost(returnJson.getBigDecimal("event_order_paid_cost"));
            reportHour.setEventNextDayStay(returnJson.getLong("event_next_day_stay"));
            reportHour.setEventNextDayStayCost(returnJson.getBigDecimal("event_next_day_stay_cost"));
            reportHour.setEventNextDayStayRatio(returnJson.getBigDecimal("event_next_day_stay_ratio"));
            reportHour.setFormCount(returnJson.getLong("form_count"));
            reportHour.setFormCost(returnJson.getBigDecimal("form_cost"));
            reportHour.setFormActionRatio(returnJson.getBigDecimal("form_action_ratio"));
            reportHour.setEventJinJianLandingPage(returnJson.getLong("event_jin_jian_landing_page"));
            reportHour.setEventJinJianLandingPageCost(returnJson.getBigDecimal("event_jin_jian_landing_page_cost"));
            reportHour.setEventCreditGrantLandingPage(returnJson.getLong("event_credit_grant_landing_page"));
            reportHour.setEventCreditGrantLandingPageCost(returnJson.getBigDecimal("event_credit_grant_landing_page_cost"));
            reportHour.setEventCreditGrantLandingPageRatio(returnJson.getBigDecimal("event_credit_grant_landing_page_ratio"));
            reportHour.setEventValidClues(returnJson.getLong("event_valid_clues"));
            reportHour.setEventValidCluesCost(returnJson.getBigDecimal("event_valid_clues_cost"));
            reportHour.setEventAddWechat(returnJson.getLong("event_add_wechat"));
            reportHour.setEventAddWechatCost(returnJson.getBigDecimal("event_add_wechat_cost"));
            reportHour.setEventAddWechatRatio(returnJson.getBigDecimal("event_add_wechat_ratio"));
            reportHour.setEventGetThrough(returnJson.getLong("event_get_through"));
            reportHour.setEventGetThroughCost(returnJson.getBigDecimal("event_get_through_cost"));
            reportHour.setEventGetThroughRatio(returnJson.getBigDecimal("event_get_through_ratio"));
            reportHour.setEventAppInvoked(returnJson.getLong("event_app_invoked"));
            reportHour.setEventAppInvokedCost(returnJson.getBigDecimal("event_app_invoked_cost"));
            reportHour.setEventAppInvokedRatio(returnJson.getBigDecimal("event_app_invoked_ratio"));
            reportHour.setEventCreditGrantLandingRatio(returnJson.getBigDecimal("event_credit_grant_landing_ratio"));
            reportHour.setPlay5sRatio(returnJson.getBigDecimal("play_5s_ratio"));
            reportHour.setPlayEndRatio(returnJson.getBigDecimal("play_end_ratio"));
            reportHour.setEventNewUserPay(returnJson.getBigDecimal("event_new_user_pay"));
            reportHour.setEventNewUserPayCost(returnJson.getBigDecimal("event_new_user_pay_cost"));
            reportHour.setEventNewUserPayRatio(returnJson.getBigDecimal("event_new_user_pay_ratio"));
            reportHour.setClick1kCost(returnJson.getBigDecimal("click_1k_cost"));
            reportHour.setAdProductCnt(returnJson.getLong("ad_product_cnt"));
            reportHour.setEventGoodsView(returnJson.getLong("event_goods_view"));
            reportHour.setMerchantRecoFans(returnJson.getLong("merchant_reco_fans"));
            reportHour.setEventGoodsViewCost(returnJson.getBigDecimal("event_goods_view_cost"));
            reportHour.setMerchantRecoFansCost(returnJson.getBigDecimal("merchant_reco_fans_cost"));
            reportHour.setEventOrderAmountRoi(returnJson.getBigDecimal("event_order_amount_roi"));
            reportHour.setEventButtonClick(returnJson.getLong("event_button_click"));
            reportHour.setEventButtonClickCost(returnJson.getBigDecimal("event_button_click_cost"));
            reportHour.setEventButtonClickRatio(returnJson.getBigDecimal("event_button_click_ratio"));
            reportHour.setEventOrderPaidRoi(returnJson.getBigDecimal("event_order_paid_roi"));
            reportHour.setEventNewUserJinjianApp(returnJson.getLong("event_new_user_jinjian_app"));
            reportHour.setEventNewUserJinjianAppCost(returnJson.getBigDecimal("event_new_user_jinjian_app_cost"));
            reportHour.setEventNewUserJinjianAppRoi(returnJson.getBigDecimal("event_new_user_jinjian_app_roi"));
            reportHour.setEventNewUserCreditGrantApp(returnJson.getLong("event_new_user_credit_grant_app"));
            reportHour.setEventNewUserCreditGrantAppCost(returnJson.getBigDecimal("event_new_user_credit_grant_app_cost"));
            reportHour.setEventNewUserCreditGrantAppRoi(returnJson.getBigDecimal("event_new_user_credit_grant_app_roi"));
            reportHour.setEventNewUserJinjianPage(returnJson.getLong("event_new_user_jinjian_page"));
            reportHour.setEventNewUserJinjianPageCost(returnJson.getBigDecimal("event_new_user_jinjian_page_cost"));
            reportHour.setEventNewUserJinjianPageRoi(returnJson.getBigDecimal("event_new_user_jinjian_page_roi"));
            reportHour.setEventNewUserCreditGrantPage(returnJson.getLong("event_new_user_credit_grant_page"));
            reportHour.setEventNewUserCreditGrantPageCost(returnJson.getBigDecimal("event_new_user_credit_grant_page_cost"));
            reportHour.setEventNewUserCreditGrantPageRoi(returnJson.getBigDecimal("event_new_user_credit_grant_page_roi"));
            reportHour.setEventAppointForm(returnJson.getLong("event_appoint_form"));
            reportHour.setEventAppointFormCost(returnJson.getBigDecimal("event_appoint_form_cost"));
            reportHour.setEventAppointFormRatio(returnJson.getBigDecimal("event_appoint_form_ratio"));
            reportHour.setEventAppointJumpClick(returnJson.getLong("event_appoint_jump_click"));
            reportHour.setEventAppointJumpClickCost(returnJson.getBigDecimal("event_appoint_jump_click_cost"));
            reportHour.setEventAppointJumpClickRatio(returnJson.getBigDecimal("event_appoint_jump_click_ratio"));
            reportHour.setUnionEventPayPurchaseAmount7d(returnJson.getLong("union_event_pay_purchase_amount_7d"));
            reportHour.setUnionEventPayPurchaseAmount7dRoi(returnJson.getBigDecimal("union_event_pay_purchase_amount_7d_roi"));
            reportHour.setPlacementType(returnJson.getString("placement_type"));
            reportHour.setAdScene(returnJson.getString("ad_scene"));
            reportHour.setEventOrderSuccessed(returnJson.getLong("event_order_successed"));
            reportHour.setAdPhotoPlayed10sRatio(returnJson.getBigDecimal("ad_photo_played_10s_ratio"));
            reportHour.setKeyActionCost(returnJson.getBigDecimal("key_action_cost"));
            reportHour.setEventAddShoppingCartCost(returnJson.getBigDecimal("event_add_shopping_cart_cost"));
            reportHour.setEventAdWatchTimesRatio(returnJson.getBigDecimal("event_ad_watch_times_ratio"));
            reportHour.setEventOutboundCallCost(returnJson.getBigDecimal("event_outbound_call_cost"));
            reportHour.setEventOutboundCall(returnJson.getLong("event_outbound_call"));
            reportHour.setActionNewRatio(returnJson.getBigDecimal("action_new_ratio"));
            reportHour.setEventWatchAppAd(returnJson.getLong("event_watch_app_ad"));
            reportHour.setEventMultiConversionCost(returnJson.getBigDecimal("event_multi_conversion_cost"));
            reportHour.setEventPhoneCardActivate(returnJson.getLong("event_phone_card_activate"));
            reportHour.setAdPhotoPlayed75percentRatio(returnJson.getBigDecimal("ad_photo_played_75percent_ratio"));
            // reportHour.setKeyActionRatio(returnJson.getBigDecimal("key_action_ratio"));
            reportHour.setEventAdWatchTimesCost(returnJson.getBigDecimal("event_ad_watch_times_cost"));
            reportHour.setEventAddShoppingCart(returnJson.getLong("event_add_shopping_cart"));
            reportHour.setKeyAction(returnJson.getLong("key_action"));
            reportHour.setEventMultiConversion(returnJson.getLong("event_multi_conversion"));
            reportHour.setEventWechatConnected(returnJson.getLong("event_wechat_connected"));
            reportHour.setEventDspGiftForm(returnJson.getLong("event_dsp_gift_form"));
            reportHour.setEventIntentionConfirmed(returnJson.getLong("event_intention_confirmed"));
            reportHour.setEventPhoneGetThrough(returnJson.getLong("event_phone_get_through"));
            reportHour.setEventMultiConversionRatio(returnJson.getBigDecimal("event_multi_conversion_ratio"));
            reportHour.setEventMeasurementHouse(returnJson.getBigDecimal("event_measurement_house"));
            reportHour.setAdPhotoPlayed2sRatio(returnJson.getBigDecimal("ad_photo_played_2s_ratio"));
            reportHour.setEventOutboundCallRatio(returnJson.getBigDecimal("event_outbound_call_ratio"));
            reportHour.setEventAdWatchTimes(returnJson.getLong("event_ad_watch_times"));
            addList.add(reportHour);
        }
        unitReportHourlyMapper.replaceBatch(addList);
        if (toGet) {
            getUnitReportHour(advertiserId, accessToken, startDate, endDate, page + 1);
        } else {
            log.info("组时报数据获取完成：accountId：{}", advertiserId);
        }

    }
}
