package cn.com.ctop.job.kuaishou.data.service.impl;

import cn.com.ctop.job.kuaishou.data.entity.MediaSendAlarmRecord;
import cn.com.ctop.job.kuaishou.data.mapper.AdUnitReportMapper;
import cn.com.ctop.job.kuaishou.data.service.IAdUnitReportService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import cn.com.ctop.job.kuaishou.data.service.IWarningOperationService;
import cn.com.ctop.job.kuaishou.data.service.IWexinConfigService;
import cn.com.ctop.job.kuaishou.data.utils.weixin.CorpWexinUtils;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@Service
public class WarningOperationServiceImpl implements IWarningOperationService {
    @Autowired
    private IOauthTokenService tokenService;
    @Autowired
    private IAdUnitReportService unitReportService;
    @Resource
    private AdUnitReportMapper adUnitReportMapper;
    @Autowired
    private CorpWexinUtils corpWexinUtils;
    @Resource
    private IWexinConfigService wexinService;

    /**
     * 快手出价方式预警 && 操作
     *
     * @param projectId
     * @param projectName
     * @param accountId
     * @param unitId
     * @param unit_name
     */

    @Override
    public void kuaiShouWarningOperation(Long projectId, String projectName, Long accountId, Long unitId, String unit_name, String msg) {
        String token = tokenService.getByAccountId(accountId);
        unitReportService.updateUnitStatus(token, accountId, unitId, 2);
        String text = getKuaiShouBidTypeMessage(projectName + "（" + projectId + "）", accountId, unitId, unit_name, msg);
        //日志记录
        MediaSendAlarmRecord record = new MediaSendAlarmRecord(accountId, "2", text, unitId);
        adUnitReportMapper.insertSendRecord(record);

        List<JSONObject> wexinIds = wexinService.selectUsersByAccountId(accountId);
        corpWexinUtils.sendMessage(wexinIds, text);
    }

    private String getKuaiShouBidTypeMessage(String projectName, Long advertiserId, Long unitId, String unit_name, String msg) {
        StringBuilder text = new StringBuilder();
        text.append("出价方式错误预警").append("<br/>")
                .append("您本地的项目：").append(projectName).append("<br/>")
                .append("下的账户：").append(advertiserId).append("<br/>")
                .append("广告组id：").append(unitId).append("<br/>")
                .append("广告组名称：").append(unit_name).append("<br/>")
                .append("预警说明：").append("<br/>")
                .append(msg).append("<br/>")
                .append("系统已暂停该广告组，请您及时查看并处理！");
        return text.toString();
    }


    /**
     * 出价 过高预警 && 操作
     *
     * @param projectId
     * @param projectName
     * @param accountId
     * @param unitId
     * @param unit_name
     */

    @Override
    public void kuaiShouBidWarningOperation(Long projectId, String projectName, Long accountId, Long unitId, String unit_name, String msg) {
        String token = tokenService.getByAccountId(accountId);
        unitReportService.updateUnitStatus(token, accountId, unitId, 2);
        //日志记录
        String text = getKuaiShouBidMessage(projectName + "（" + projectId + "）", accountId, unitId, unit_name, msg);
        MediaSendAlarmRecord record = new MediaSendAlarmRecord(accountId, "2", text, unitId);
        adUnitReportMapper.insertSendRecord(record);

        List<JSONObject> wexinIds = wexinService.selectUsersByAccountId(accountId);
        corpWexinUtils.sendMessage(wexinIds, text);
    }

    private String getKuaiShouBidMessage(String projectName, Long advertiserId, Long unitId, String unit_name, String msg) {
        StringBuilder text = new StringBuilder();
        text.append("出价过高预警").append("<br/>")
                .append("您本地的项目：").append(projectName).append("<br/>")
                .append("下的账户：").append(advertiserId).append("<br/>")
                .append("广告组id：").append(unitId).append("<br/>")
                .append("广告组名称：").append(unit_name).append("<br/>")
                .append("预警说明：").append("<br/>")
                .append(msg).append("<br/>")
                .append("系统已暂停该广告组，请您及时查看并处理！");
        return text.toString();
    }
}
