package cn.com.ctop.job.kuaishou.data.utils;

import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.CookieSpecs;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.util.EntityUtils;

import javax.net.ssl.SSLContext;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.Map;

@Slf4j
public class HttpUtils {

  public static CookieStore COOKIESTORE = new BasicCookieStore();
  /** setProxy(new HttpHost("106.125.239.179", 4245)) */
  public static Map<String, Map<String, String>> fontsMap = new HashMap<>();

  public static String USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36";

  public static CloseableHttpClient createSslClientDefault() {
    try {
      SSLContext sslContext =
          new SSLContextBuilder()
              .loadTrustMaterial(
                  null,
                  new TrustStrategy() {
                    // 信任所有证书
                    @Override
                    public boolean isTrusted(X509Certificate[] chain, String authType)
                        throws CertificateException {
                      return true;
                    }
                  })
              .build();
      RequestConfig globalConfig =
          RequestConfig.custom()
              .setConnectTimeout(30000)
              .setSocketTimeout(30000)
              .setCookieSpec(CookieSpecs.STANDARD)
              .build();
      SSLConnectionSocketFactory sslFactory = new SSLConnectionSocketFactory(sslContext);
      return HttpClients.custom()
          .setDefaultCookieStore(COOKIESTORE)
          .setDefaultRequestConfig(globalConfig)
          .setConnectionReuseStrategy((response, context) -> false)
          .setSSLSocketFactory(sslFactory)
          .build();
    } catch (Exception e) {
      log.error("处理Https证书异常", e);
    }
    return HttpClients.createDefault();
  }


  public static String kuaiShouhttpPostRequest(String url, String body, Map<String, String> headers) {
    HttpClient httpClient = createSslClientDefault();
    String strReturn = "";
    try {
      HttpPost httppost = new HttpPost(url);
      RequestConfig requestConfig = RequestConfig.custom().setSocketTimeout(120000).setConnectTimeout(120000).build();
      httppost.setConfig(requestConfig);
      if (!Check.isNullMap(headers)) {
        for (String key : headers.keySet()) {
          httppost.setHeader(key, headers.get(key));
        }
      }

      httppost.setHeader("User-Agent", USER_AGENT);
      if (!Check.isNull(body)) {
        httppost.setEntity(new StringEntity(body, Charset.forName("UTF-8")));
      }
      HttpEntity respentity;
      HttpResponse response = httpClient.execute(httppost);
      int statusCode = response.getStatusLine().getStatusCode();
      if (statusCode == HttpStatus.SC_OK) {
        respentity = response.getEntity();
        strReturn = EntityUtils.toString(respentity);
        return strReturn;
      }
    } catch (Exception e) {
      e.printStackTrace();
      log.error(e.getMessage());
    }
    return strReturn;
  }

  public static String httpPostRequest(String url, Map<String, Object> param, Map<String, String> headers) {

    HttpClient httpClient = createSslClientDefault();
    String strReturn = "";
    try {
      HttpPost httppost = new HttpPost(url);
      httppost.setHeader("User-Agent", USER_AGENT);
      httppost.addHeader("Content-Type", "application/json");
      if (headers != null && headers.size() > 0) {
        for (String key : headers.keySet()) {
          httppost.setHeader(key, headers.get(key));
        }
      }
      if (param != null && param.size() > 0) {
        httppost.setEntity(new StringEntity(new Gson().toJson(param), Charset.forName("UTF-8")));
      }
      RequestConfig requestConfig =
          RequestConfig.custom().setSocketTimeout(120000).setConnectTimeout(120000).build();
      httppost.setConfig(requestConfig);
      HttpEntity respentity;

      HttpResponse response = httpClient.execute(httppost);
      int statusCode = response.getStatusLine().getStatusCode();
      if (statusCode == HttpStatus.SC_MOVED_PERMANENTLY
          || statusCode == HttpStatus.SC_MOVED_TEMPORARILY) {
        String newUrl = response.getFirstHeader("Location").getValue();
        return httpPostRequest(newUrl, param, headers);
      } else if (statusCode == HttpStatus.SC_OK) {
        respentity = response.getEntity();
        strReturn = EntityUtils.toString(respentity);
        return strReturn;
      }
    } catch (Exception e) {
      e.printStackTrace();
      log.error("httpPostRequest接口异常:"+e.getMessage());
    }
    return strReturn;
  }


  public static String httpGet(String url, Map<String, Object> paramsMap, Map<String, String> headers) {
    String result = null;
    try {
      // CloseableHttpClient httpClient = HttpClients.createDefault();
      //创建参数列表
      HttpClient httpclient = getHttpclient();
      StringBuilder postBody = null;
      if (!Check.isNull(paramsMap)) {
        postBody = new StringBuilder();
        for (Map.Entry<String, Object> entry : paramsMap.entrySet()) {
          if (entry.getValue() == null) {
            continue;
          }
          postBody.append(entry.getKey()).append("=").append(URLEncoder.encode(entry.getValue().toString(),
                  "utf-8")).append("&");
        }
        if (!paramsMap.isEmpty()) {
          postBody.deleteCharAt(postBody.length() - 1);
        }
      }
      String urlStr;
      if (!Check.isNull(postBody)) {
        urlStr = url + "?" + postBody;
      } else {
        urlStr = url;
      }
      HttpGet httpget = new HttpGet(urlStr);
      httpget.setHeader("User-Agent", USER_AGENT);
      if (!Check.isNullMap(headers)) {
        for (String key : headers.keySet()) {
          httpget.setHeader(key, headers.get(key));
        }
      }
      HttpEntity respEntity;
      HttpResponse response = httpclient.execute(httpget);
      respEntity = response.getEntity();
      result = EntityUtils.toString(respEntity);
    } catch (Exception e) {
      log.error("http get请求异常");
      e.printStackTrace();
    }
    return result;
  }


  public static HttpClient getHttpclient() {
    RequestConfig defaultRequestConfig = RequestConfig.custom().setSocketTimeout(180000).setConnectTimeout(30000)
            .setConnectionRequestTimeout(30000).setStaleConnectionCheckEnabled(true).build();
    return HttpClients.custom().setDefaultRequestConfig(defaultRequestConfig).build();
  }
}
