package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportHourlyService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Component
public class AccountReportlJob {
    @Autowired
    private IOauthTokenService tokenService;
    @Autowired
    private IKuaishouAccountReportDailyService kuaishouAccountReportDailyService;
    @Autowired
    private IKuaishouAccountReportHourlyService kuaishouAccountReportHourlyService;
    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);

    /**
     * 获取前天~昨天账户日报
     *
     * @throws Exception
     */
    @XxlJob("accountDailyReport")
    public void accountDailyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String endDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        String startDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        try {
                            kuaishouAccountReportDailyService.getAdvertiserReportDaily(Long.valueOf(param), token, startDate, endDate, 1);
                        } catch (Exception exception) {
                            exception.printStackTrace();
                        }
                    }
                });
    }

    /**
     * 获取当天账户时报
     *
     * @throws Exception
     */
    @XxlJob("accountNowDateHourReport")
    public void accountNowDateHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        kuaishouAccountReportHourlyService.getAdvertiserReportHour(
                                Long.valueOf(param), token, nowDate, nowDate, 1);
                    }
                });
    }

    /**
     * 获取昨天账户时报
     *
     * @throws Exception
     */
    @XxlJob("accountYesterdayHourReport")
    public void accountYesterdayHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterday = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        kuaishouAccountReportHourlyService.getAdvertiserReportHour(
                                Long.valueOf(param), token, yesterday, yesterday, 1);
                    }
                });
    }

    /**
     * 获取昨天账户时报
     *
     * @throws Exception
     */
    @XxlJob("accountBeforeYesterdayHourReport")
    public void accountBeforeYesterdayHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String beforeYesterday = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        kuaishouAccountReportHourlyService.getAdvertiserReportHour(
                                Long.valueOf(param), token, beforeYesterday, beforeYesterday, 1);
                    }
                });
    }

    /**
     * 获取原生效果数据整体概览接口（数据概览、分日转化数据分布、转化漏斗）
     * 近三天
     */
    @XxlJob("effectNativeChartReport")
    public void effectNativeChartReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }

        //当天
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        //昨天
        String yesterday = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        //前天
        String beforeYesterday = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        //大前天
        String beforeYesterday2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -3);

        dailyExecutorService.submit(
                new Runnable() {
                    @SneakyThrows
                    @Override
                    public void run() {
                        Thread.sleep(500);
                        kuaishouAccountReportDailyService.effectNativeChartReport(Long.valueOf(param), token, yesterday, yesterday);
                        kuaishouAccountReportDailyService.effectNativeChartReport(Long.valueOf(param), token, beforeYesterday, beforeYesterday);
                        kuaishouAccountReportDailyService.effectNativeChartReport(Long.valueOf(param), token, beforeYesterday2, beforeYesterday2);

                    }
                });
    }
}
