package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.service.IKuaishouCampaignReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouCampaignReportHourlyService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Component
public class CampaignReportJob {
  @Autowired private IOauthTokenService tokenService;
  private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
  private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
  @Autowired private IKuaishouCampaignReportDailyService campaignReportDailyService;
  @Autowired private IKuaishouCampaignReportHourlyService campaignReportHourlyService;

  /**
   * 获取前天~昨天账户日报
   *
   * @throws Exception
   */
  @XxlJob("campaignDailyReport")
  public void campaignDailyReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
    dailyExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            campaignReportDailyService.getCampaignReportDaily(
                Long.valueOf(param), token, date, date, 1);
          }
        });
  }

  /**
   * 获取前天~当天账户日报
   *
   * @throws Exception
   */
  @XxlJob("campaignToDailyReport")
  public void campaignToDailyReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    dailyExecutorService.submit(
            new Runnable() {
              @Override
              public void run() {
                campaignReportDailyService.getCampaignReportDaily(
                        Long.valueOf(param), token, nowDate, nowDate, 1);
              }
            });
  }

  /**
   * 获取前天账户日报
   *
   * @throws Exception
   */
  @XxlJob("campaignBeforeYesterdayDailyReport")
  public void campaignBeforeYesterdayDailyReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
    dailyExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            campaignReportDailyService.getCampaignReportDaily(
                Long.valueOf(param), token, date, date, 1);
          }
        });
  }

  /**
   * 当日计划时报
   *
   * @throws Exception
   */
  @XxlJob("campaignNowDateHourReport")
  public void campaignNowDateHourReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    hourExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            campaignReportHourlyService.getCampaignReportHour(
                Long.valueOf(param), token, nowDate, nowDate, 1);
          }
        });
  }

  /**
   * 昨日计划日报
   *
   * @throws Exception
   */
  @XxlJob("campaignYesterdayHourReport")
  public void campaignYesterdayHourReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
    hourExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            campaignReportHourlyService.getCampaignReportHour(
                Long.valueOf(param), token, date, date, 1);
          }
        });
  }
}
