package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.service.IAdUnitReportService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouUnitReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Component
public class KuaishouGroupJob {
    @Autowired
    private IOauthTokenService tokenService;
    @Autowired
    private IAdUnitReportService unitReportService;

    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);

    /**
     * 项目转化目标、转化出价预警
     *
     * @throws Exception
     */
    @XxlJob("todayOcpxActionTypeTrackUrlJob")
    public void todayOcpxActionTypeTrackUrlJob() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String today = DateUtils.getNowDate("yyyy-MM-dd");
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        unitReportService.checkGroupInfo(Long.valueOf(param), token, today, today, 1);
                    }
                });
    }

    /**
     * 项目转化目标、转化出价预警
     *
     * @throws Exception
     */
    @XxlJob("yesterdayOcpxActionTypeTrackUrlJob")
    public void yesterdayOcpxActionTypeTrackUrlJob() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String today = DateUtils.getNowDate("yyyy-MM-dd");
        String yesterday = DateUtils.getAnotherDay("yyyy-MM-dd", DateUtils.getNowDate("yyyy-MM-dd"), -1);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        unitReportService.checkGroupInfo(Long.valueOf(param), token, yesterday, yesterday, 1);
                    }
                });
    }

}
