package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.service.IAccountWarningService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import com.alibaba.fastjson.JSONObject;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.util.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;


/**
 * @author 于蒙
 */
@Component
public class KuaishouHourlyAccountWarningLoadJob {
    @Autowired
    private IOauthTokenService tokenService;
    @Autowired
    private IAccountWarningService accountWarningService;
    static ExecutorService executorService = Executors.newFixedThreadPool(3);

    /**
     * 快手时段消耗预警
     *
     * @throws Exception
     */
    @XxlJob("kuaishouHourlyAccountWarning")
    public void execute() throws Exception {
        XxlJobHelper.log("开始时段消耗预警定时任务");
        long l = System.currentTimeMillis();
        List<JSONObject> tokens = tokenService.selectKuaiShouToken();
        if (null == tokens || tokens.size() <= 0) {
            XxlJobHelper.log("定时获取快手数据异常：未获取到可用的token");
            XxlJobHelper.handleFail();
            return;
        }
        String todayStr = DateUtil.formatDate(new Date());
        Integer todayInt = DateUtil.getIntegerDate(todayStr);
        Integer yesterdayInt = DateUtil.getIntegerDate(DateUtil.getAnotherDay("yyyy-MM-dd", todayStr, -1));
        tokens.forEach(token -> executorService.submit(() -> accountWarningService.kuaiShouAccountWarning(token.getLong("accountId"), todayInt, yesterdayInt)));
        XxlJobHelper.log("异步时段消耗预警定时任务执行完毕，耗时：{} ms", System.currentTimeMillis() - l);
    }
}
