package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouAccountReportHourlyService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import cn.com.ctop.job.kuaishou.data.service.IOperationRecordService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Component
public class OperationRecordJob {
  @Autowired
  private IOauthTokenService tokenService;

  @Autowired
  private IOperationRecordService operationRecordService;

  private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
  private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);

  /**
   * 快手操作记录
   *
   * @throws Exception
   */
  @XxlJob("getOperationRecordList")
  public void accountDailyReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }

    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String endDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
    String startDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
    dailyExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            operationRecordService.getOperationRecordList(token,Long.valueOf(param), startDate, endDate, 1);
          }
        });
  }

}
