package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.service.IKuaishouProgramCreativeReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Component
public class ProgramCreativeReportJob {
  @Autowired private IOauthTokenService tokenService;
  private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
  @Autowired private IKuaishouProgramCreativeReportDailyService programCreativeReportDailyService;

  /**
   * 获取昨天程序化创意日报
   *
   * @throws Exception
   */
  @XxlJob("programCreativeDailyReport")
  public void programCreativeDailyReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
    dailyExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            programCreativeReportDailyService.getProgramCreativeReportDaily(
                Long.valueOf(param), token, date, date, 1);
          }
        });
  }

  /**
   * 获取前天程序化创意日报
   *
   * @throws Exception
   */
  @XxlJob("programCreativeBeforeYesterdayDailyReport")
  public void programCreativeBeforeYesterdayDailyReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
    dailyExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            programCreativeReportDailyService.getProgramCreativeReportDaily(
                Long.valueOf(param), token, date, date, 1);
          }
        });
  }
}
