package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.data.utils.HttpUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

@Slf4j
@Component
public class RuixuanJob {

    private static final String RX_IP = "http://ruixuan.api.tjyourong.com.cn/";
    private static final String COMMON_IP = ".api.tjyourong.com.cn/isv/supply_chain/";
    private static final String SHOP_IP = ".api.tjyourong.com.cn/isv/supply_chain/insertShopInfo";

    /**
     * 昨天的数据 kwaixiaodian_order_cursor_list
     */
    @XxlJob("querykwaiOrderList")
    public void querykwaiOrderList() {
        try {
            String param = XxlJobHelper.getJobParam(); // 执行参数
            if (Check.isNull(param)) {
                log.error("入参为空");
                return;
            }
            String today = DateUtils.getNowDate("yyyy-MM-dd");
            String yesterday = DateUtils.getAnotherDay("yyyy-MM-dd", today, -1);

            //queryType:1按创建时间查找 2按更新时间查找 ，默认创建时间

            Map<String, Object> createMap = new HashMap<>();
            createMap.put("shopId", param);
            createMap.put("queryType", 1);
            createMap.put("beginDate", yesterday);
            createMap.put("endDate", yesterday);

            //获取订单列表
            String listUrl = RX_IP + "kwaixiaodian/item/querykwaiOrderList";
            HttpUtils.httpGet(listUrl, createMap, null);

            /* -----------获取售后单列表 ↓↓↓ begin*/

            Thread.sleep(10000);
            //获取售后单列表
            String rxAfterIp = RX_IP + "kwaixiaodian/item/afterSalesList";
            HttpUtils.httpGet(rxAfterIp, createMap, null);
            /* -----------获取售后单列表 ↑↑↑ end*/

            Thread.sleep(10000);
            Map<String, Object> updateMap = new HashMap<>();
            updateMap.put("shopId", param);
            updateMap.put("queryType", 2);
            updateMap.put("beginDate", yesterday);
            updateMap.put("endDate", yesterday);
            HttpUtils.httpGet(listUrl, updateMap, null);

            /* -----------获取售后单列表 ↓↓↓ begin*/
            Thread.sleep(10000);
            HttpUtils.httpGet(rxAfterIp, updateMap, null);
            /* -----------获取售后单列表 ↑↑↑ end*/

        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取两小时内的数据 kwaixiaodian_order_cursor_list
     */
    @XxlJob("queryNowdayKWaiOrderList")
    public void queryNowDaykwaiOrderList() {
        try {
            String param = XxlJobHelper.getJobParam(); // 执行参数
            if (Check.isNull(param)) {
                log.error("入参为空");
                return;
            }
            Map<String, Object> createMap = new HashMap<>();
            createMap.put("shopId", param);

            //获取订单列表
            String listUrl = RX_IP + "kwaixiaodian/item/queryNowdayKWaiOrderList";
            HttpUtils.httpGet(listUrl, createMap, null);

            /* -----------获取售后单列表 ↓↓↓ begin*/
            Thread.sleep(10000);
            //获取售后单列表
            String rxAfterIp = RX_IP + "kwaixiaodian/item/queryNowdayafterSalesList";
            HttpUtils.httpGet(rxAfterIp, createMap, null);
            /* -----------获取售后单列表 ↑↑↑ end*/
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

    }

    /**
     * 获取90天内的数据 kwaixiaodian_order_cursor_list
     */
    @XxlJob("querykwaiOrderListMonth")
    public void querykwaiOrderListMonth() {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        Map<String, Object> createMap = new HashMap<>();
        createMap.put("shopId", param);

//        //获取订单列表
//        String listUrl = RX_IP + "kwaixiaodian/item/querykwaiOrderListMonth";
//
        //获取售后单列表
        String listUrl = RX_IP + "kwaixiaodian/item/queryAfterSalesListMonth";
        HttpUtils.httpGet(listUrl, createMap, null);
    }


    /**
     * 店铺体验分、带货口碑分 kwaixiaodian_shop_score
     */
    @XxlJob("queryShopScore")
    public void queryShopScore() {
        // 执行参数
        String param = XxlJobHelper.getJobParam();
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        Map<String, Object> createMap = new HashMap<>();
        createMap.put("shopId", param);

        //获取带货口碑分信息
        String url = RX_IP + "kwaixiaodian/item/queryMasterScore";
        HttpUtils.httpGet(url, createMap, null);

        //获取店铺体验分信息
        String Url2 = RX_IP + "kwaixiaodian/item/queryShopScore";
        HttpUtils.httpGet(Url2, createMap, null);

    }





    /**
     * 插入昨天数据 kuaishou_supply_chain
     */
    @XxlJob("insertOrderList")
    public void insertOrderList() {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String[] arr = param.split("_");
        if (arr.length > 1) {
            String dbType = arr[0];
            String tokenId = arr[1];
            String url = null;
            if ("miaogousi".equals(dbType)) {
                url = "http://mgs" + COMMON_IP + "insertOrderList";
            } else {
                url = "http://" + dbType + COMMON_IP + "insertOrderList";
            }
            log.info("---tokenId:{},订单URL:{}", tokenId, url);
            String today = DateUtils.getNowDate("yyyy-MM-dd");
            String yesterday = DateUtils.getAnotherDay("yyyy-MM-dd", today, -1);

            Map<String, Object> createMap = new HashMap<>();
            createMap.put("tokenId", tokenId);
            createMap.put("queryType", 1);
            createMap.put("beginDate", yesterday);
            createMap.put("endDate", yesterday);
            //获取订单列表
            HttpUtils.httpGet(url, createMap, null);

            Map<String, Object> updateMap = new HashMap<>();
            updateMap.put("tokenId", tokenId);
            updateMap.put("queryType", 2);
            updateMap.put("beginDate", yesterday);
            updateMap.put("endDate", yesterday);
            HttpUtils.httpGet(url, updateMap, null);
        }
    }

    /**
     * 获取两小时内的数据 kuaishou_supply_chain
     */
    @XxlJob("insertNowdayOrderList")
    public void insertNowdayOrderList() {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String[] arr = param.split("_");
        if (arr.length > 1) {
            String dbType = arr[0];
            String tokenId = arr[1];
            String url = null;
            if ("miaogousi".equals(dbType)) {
                url = "http://mgs" + COMMON_IP + "insertNowdayOrderList";
            } else {
                url = "http://" + dbType + COMMON_IP + "insertNowdayOrderList";
            }

            //获取订单列表
            Map<String, Object> createMap = new HashMap<>();
            createMap.put("tokenId", tokenId);
            HttpUtils.httpGet(url, createMap, null);
            log.info("---tokenId:{},订单URL:{}", tokenId, url);
        }
    }

    /**
     * 插入店铺名称
     * 昨天+今天 kuaishou_activity_shop_info
     */
    @XxlJob("insertShopInfo")
    public void insertShopInfo() {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String[] arr = param.split("_");
        if (arr.length > 1) {
            String dbType = arr[0];
            String tokenId = arr[1];
            String url = null;
            if ("miaogousi".equals(dbType)) {
                url = "http://mgs" + SHOP_IP;
            } else {
                url = "http://" + dbType + SHOP_IP;
            }
            log.info("---tokenId:{},订单URL:{}", tokenId, url);
            Map<String, Object> createMap = new HashMap<>();
            createMap.put("tokenId", tokenId);
            createMap.put("bdtype", dbType);
            //录入店铺名称
            HttpUtils.httpGet(url, createMap, null);
        }
    }

}
