package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.service.IKuaishouUnitReportDailyService;
import cn.com.ctop.job.kuaishou.data.service.IKuaishouUnitReportHourlyService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.alibaba.fastjson.JSONObject;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Component
public class UnitReportJob {
    @Autowired
    private IOauthTokenService tokenService;
    private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService adSceneDailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService adSceneDailyExecutorService2 = Executors.newFixedThreadPool(3);
    private static ExecutorService placementTypeDailyExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
    @Autowired
    private IKuaishouUnitReportDailyService unitReportDailyService;
    @Autowired
    private IKuaishouUnitReportHourlyService unitReportHourlyService;

    /**
     * 获取昨天组日报
     *
     * @throws Exception
     */
    @XxlJob("unitDailyReport")
    public void unitDailyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        unitReportDailyService.getUnitReportDaily(Long.valueOf(param), token, date, date, 1);
                    }
                });
    }

    /**
     * 获取昨天组日报_按广告场景
     *
     * @throws Exception
     */
    @XxlJob("unitAdSceneTypeDailyReport")
    public void unitAdSceneTypeDailyReport() throws Exception {
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String startDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        List<JSONObject> tokens = tokenService.getByProjectId(458L);    //淘特项目
        tokens.forEach(t -> adSceneDailyExecutorService.submit(() -> {
            unitReportDailyService.getUnitAdSceneReportDaily(t.getLong("accountId"), t.getString("accessToken"), startDate, startDate, 1);
        }));
        String endDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        tokens.forEach(t -> adSceneDailyExecutorService2.submit(() -> {
            try {
                Thread.sleep(500L);
                unitReportDailyService.getUnitAdSceneReportDaily(t.getLong("accountId"), t.getString("accessToken"), endDate, endDate, 1);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }));
    }

    /**
     * 获取昨天组日报_按广告范围
     *
     * @throws Exception
     */
    @XxlJob("unitPlacementTypeDailyReport")
    public void unitPlacementTypeDailyReport() throws Exception {
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        List<JSONObject> tokens = tokenService.getByProjectId(458L);    //淘特项目
        tokens.forEach(t -> placementTypeDailyExecutorService.submit(() -> {
            unitReportDailyService.getUnitPlacementTypeReportDaily(t.getLong("accountId"), t.getString("accessToken"), date, date, 1);
        }));
    }

    /**
     * 获取前天组日报
     *
     * @throws Exception
     */
    @XxlJob("unitBeforeYesterdayDailyReport")
    public void unitBeforeYesterdayDailyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
        dailyExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        unitReportDailyService.getUnitReportDaily(Long.valueOf(param), token, date, date, 1);
                    }
                });
    }

    /**
     * 当日组时报
     *
     * @throws Exception
     */
    @XxlJob("unitNowDateHourReport")
    public void unitNowDateHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        unitReportHourlyService.getUnitReportHour(
                                Long.valueOf(param), token, nowDate, nowDate, 1);
                    }
                });
    }

    /**
     * 昨日组时报
     *
     * @throws Exception
     */
    @XxlJob("unitYesterdayHourReport")
    public void unitYesterdayHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        hourExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        unitReportHourlyService.getUnitReportHour(Long.valueOf(param), token, date, date, 1);
                    }
                });
    }
}
