package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.service.IKuaishouVideoListService;
import cn.com.ctop.job.kuaishou.data.service.IOauthTokenService;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import cn.com.ctop.job.kuaishou.utils.RedisUtil;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Component
public class VideoListJob {
    @Autowired
    private IOauthTokenService tokenService;
    private static ExecutorService nowExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService yesterdayExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService allExecutorService = Executors.newFixedThreadPool(3);
    private static ExecutorService videoRelateCreateCountExecutorService = Executors.newFixedThreadPool(8);
    @Autowired
    private IKuaishouVideoListService videoListService;


    /**
     * 当日视频物料
     *
     * @throws Exception
     */
    @XxlJob("nowDateVideoList")
    public void videoDailyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        nowExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        videoListService.getVideoList(Long.valueOf(param), token, nowDate, nowDate, 1);
                    }
                });
    }

    /**
     * 昨日视频物料
     *
     * @throws Exception
     */
    @XxlJob("yesterdayVideoList")
    public void videoBeforeYesterdayDailyReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
        yesterdayExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        videoListService.getVideoList(Long.valueOf(param), token, date, date, 1);
                    }
                });
    }

    /**
     * 所有视频物料
     *
     * @throws Exception
     */
    @XxlJob("allVideoList")
    public void videoNowDateHourReport() throws Exception {
        String param = XxlJobHelper.getJobParam(); // 执行参数
        if (Check.isNull(param)) {
            log.error("入参为空");
            return;
        }
        String token = tokenService.getByAccountId(Long.valueOf(param));
        if (Check.isNull(token)) {
            log.error("此账户未获取到相关token，accountId：{}", param);
            return;
        }
        allExecutorService.submit(
                new Runnable() {
                    @Override
                    public void run() {
                        videoListService.getVideoList(Long.valueOf(param), token, null, null, 1);
                    }
                });
    }


    @Autowired
    private RedisUtil redisUtil;

    /**
     * 所有视频物料
     *
     * @throws Exception
     */
    @XxlJob("allVideoListSupplement")
    public void allVideoListSupplement() throws Exception {
        //获取视频物料信息异常的key
        Set<String> set = redisUtil.kesy("kuaiShou:allVideo:list*");
        if (Check.isNull(set)) {
            log.info("快手视频数据暂时无遗漏---》》》日期：{}", DateUtils.getNowDate("yyyy-MM-dd"));
            return;
        }
        List<String> list = new ArrayList<>(set);
        for (String key : list) {
            //添加重试
            List<Object> accountList;
            accountList = redisUtil.getList(key, null);
            //遍历key下的所有信息
            for (Object accountId : accountList) {
                log.info("快手视频遗漏数据补充,accountId:{}", accountId);
                String token = tokenService.getByAccountId(Long.valueOf(String.valueOf(accountId)));
                if (Check.isNull(token)) {
                    log.error("此账户未获取到相关token，accountId：{}", accountId);
                    return;
                }

                videoListService.getVideoList(Long.valueOf(accountId.toString()), token, null, null, 1);
                accountList = redisUtil.getList(key, null);

                if (accountList.size() == 0) {
                    redisUtil.delete(key);
                }

                try {
                    Thread.sleep(1000 * 2);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

            }

        }
    }


    /**
     * 支付宝-拉活-快手
     * 支付宝-商家码
     * 支付宝-激活
     * 快手-瓜子二手车
     * 瓜子-专项
     * 百度极速版
     * 易车
     * 易车汽车报价
     * 快手-京东超市-拉新
     */
    @XxlJob("getVideoRelateCreateCount")
    public void getVideoRelateCreateCount() throws Exception {
        List<Long> projectIds = new ArrayList<>();
        projectIds.add(50L);
        projectIds.add(67L);
        projectIds.add(123L);
        projectIds.add(2400915L);
        projectIds.add(7920890L);
        projectIds.add(12330885L);
        projectIds.add(14310886L);
        projectIds.add(14490902L);
        projectIds.add(15120886L);
        projectIds.add(15750885L);

        String endTime = DateUtils.getNowDate("yyyy-MM-dd");
        String startTime = DateUtils.getAnotherDay("yyyy-MM-dd", endTime, -31);

//        String endTime = null;
//        String startTime = null;

        for (Long projectId : projectIds) {
            List<Long> accountIds = videoListService.getAccountIds(projectId);
            if (!Check.isNull(accountIds)) {
                accountIds.forEach(accountId -> videoRelateCreateCountExecutorService.submit(() -> {
                    videoListService.getVideosByAccountId(accountId, startTime, endTime);
                }));
            }
        }
    }

}
