package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.data.service.*;
import cn.com.ctop.job.kuaishou.data.utils.Check;
import cn.com.ctop.job.kuaishou.data.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Component
public class VideoReportJob {
  @Autowired private IOauthTokenService tokenService;
  private static ExecutorService dailyExecutorService = Executors.newFixedThreadPool(3);
  private static ExecutorService hourExecutorService = Executors.newFixedThreadPool(3);
  @Autowired private IKuaishouMaterialVideoReportDailyService videoReportDailyService;
  @Autowired private IKuaishouMaterialVideoReportHourlyService videoReportHourlyService;


  /**
   * 获取昨天视频日报
   *
   * @throws Exception
   */
  @XxlJob("videoDailyReport")
  public void videoDailyReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String startDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
    String endDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -5);
    dailyExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            videoReportDailyService.getVideoReportDaily(Long.valueOf(param), token, startDate, startDate, 1);
          }
        });
  }

  /**
   * 获取前天视频日报
   *
   * @throws Exception
   */
  @XxlJob("videoBeforeYesterdayDailyReport")
  public void videoBeforeYesterdayDailyReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
    dailyExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            videoReportDailyService.getVideoReportDaily(Long.valueOf(param), token, date, date, 1);
          }
        });
  }

  /**
   * 当日视频时报
   *
   * @throws Exception
   */
  @XxlJob("videoNowDateHourReport")
  public void videoNowDateHourReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    hourExecutorService.submit(
        new Runnable() {
          @Override
          public void run() {
            videoReportHourlyService.getVideoReportHour(Long.valueOf(param), token, nowDate, nowDate, 1);
          }
        });
  }

  /**
   * 昨日视频时报
   *
   * @throws Exception
   */
  @XxlJob("videoYesterdayHourReport")
  public void unitYesterdayHourReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
    hourExecutorService.submit(
        new Runnable() {
          @Override
         public void run() {
            videoReportHourlyService.getVideoReportHour(Long.valueOf(param), token, date, date, 1);
          }
        });
  }




  /**
   * 图片素材日报
   *
   * @throws Exception
   */
  @XxlJob("imageDateDailyReport")
  public void imageDateDailyReport() throws Exception {
    String param = XxlJobHelper.getJobParam(); // 执行参数
    if (Check.isNull(param)) {
      log.error("入参为空");
      return;
    }
    String token = tokenService.getByAccountId(Long.valueOf(param));
    if (Check.isNull(token)) {
      log.error("此账户未获取到相关token，accountId：{}", param);
      return;
    }
    String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
    String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
    hourExecutorService.submit(
            new Runnable() {
              @Override
              public void run() {
                try {
                  videoReportDailyService.getImageReportDaily(Long.valueOf(param), token, date, nowDate, 1);
                }catch (Exception e){
                  e.printStackTrace();
                }
              }
            });
  }





}
